/**************************************************************************//**
 * @file     pga.h
 * @version  V1.00
 * $Revision: 5 $
 * $Date: 16/04/06 2:56p $
 * @brief    NM1120 PGA driver header file
 *
 * @note
 * Copyright (C) 2016 Nuvoton Technology Corp. All rights reserved.
 *****************************************************************************/
#ifndef __PGA_H__
#define __PGA_H__

#ifdef __cplusplus
extern "C"
{
#endif


/** @addtogroup NM1120_Device_Driver NM1120 Device Driver
  @{
*/

/** @addtogroup NM1120_PGA_Driver PGA Driver
  @{
*/

/** @addtogroup NM1120_PGA_EXPORTED_CONSTANTS PGA Exported Constants
  @{
*/

#define PGA_GAIN_1  (0UL)   /*!< PGA Gain mask with gain level x1    */
#define PGA_GAIN_2  (1UL)   /*!< PGA Gain mask with gain level x2    */
#define PGA_GAIN_4  (2UL)   /*!< PGA Gain mask with gain level x4    */
#define PGA_GAIN_6  (3UL)   /*!< PGA Gain mask with gain level x6    */
#define PGA_GAIN_8  (4UL)   /*!< PGA Gain mask with gain level x8    */
#define PGA_GAIN_10 (5UL)   /*!< PGA Gain mask with gain level x10   */
#define PGA_GAIN_12 (6UL)   /*!< PGA Gain mask with gain level x12   */
#define PGA_GAIN_16 (7UL)   /*!< PGA Gain mask with gain level x16   */

/*@}*/ /* end of group NM1120_PGA_EXPORTED_CONSTANTS */


/** @addtogroup NM1120_PGA_EXPORTED_FUNCTIONS PGA Exported Functions
  @{
*/

/**
  * @brief      Set PGA gain selection
  * @param[in]  pga     The pointer of the specified PGA module
  * @param[in]  u32Gain is gain index. Including :
  *             - \ref PGA_GAIN_1
  *             - \ref PGA_GAIN_2
  *             - \ref PGA_GAIN_4
  *             - \ref PGA_GAIN_6
  *             - \ref PGA_GAIN_8
  *             - \ref PGA_GAIN_10
  *             - \ref PGA_GAIN_12
  *             - \ref PGA_GAIN_16
  * @return     None
  * @details    Set PGA gain selection
  */
#define PGA_SET_GAIN(pga, u32Gain)      ((pga)->CTL = ((pga)->CTL & ~PGA_CTL_GAIN_Msk) | ((u32Gain) << PGA_CTL_GAIN_Pos))

/**
  * @brief      Get PGA gain selection bit definition.
  * @param[in]  pga     The pointer of the specified PGA module
  * @return     PGA gain selection bit definition
  * @details    Get PGA gain selection bit definition
  */
#define PGA_GET_GAIN(pga)               (((pga)->CTL | PGA_CTL_GAIN_Msk) >> PGA_CTL_GAIN_Pos)

/**
  * @brief      Enable PGA output to ACMP
  * @param[in]  pga     The pointer of the specified PGA module
  * @return     None
  * @details    This macro enable PGA output to ACMP function.
  */
#define PGA_ENABLE_OUTPUT_ACMP(pga)     ((pga)->CTL |= PGA_CTL_PGACMPEN_Msk)

/**
  * @brief      Disable PGA output to ACMP
  * @param[in]  pga     The pointer of the specified PGA module
  * @return     None
  * @details    This macro disable PGA output to ACMP function.
  */
#define PGA_DISABLE_OUTPUT_ACMP(pga)    ((pga)->CTL &= ~PGA_CTL_PGACMPEN_Msk)

/**
  * @brief      Enable PGA output to PGA_O pin
  * @param[in]  pga     The pointer of the specified PGA module
  * @return     None
  * @details    This macro enable PGA output to PGA_O pin.
  */
#define PGA_ENABLE_OUTPUT_PGAO(pga)     ((pga)->CTL |= PGA_CTL_PGAOUTEN_Msk)

/**
  * @brief      Disable PGA output to PGA_O pin
  * @param[in]  pga     The pointer of the specified PGA module
  * @return     None
  * @details    This macro disable PGA output to PGA_O pin.
  */
#define PGA_DISABLE_OUTPUT_PGAO(pga)    ((pga)->CTL &= ~PGA_CTL_PGAOUTEN_Msk)

/**
  * @brief      Enable PGA output to EADC
  * @param[in]  pga     The pointer of the specified PGA module
  * @return     None
  * @details    This macro enable PGA output to EADC.
  */
#define PGA_ENABLE_OUTPUT_EADC(pga)     ((pga)->CTL |= PGA_CTL_PGACMPEN_Msk)

/**
  * @brief      Disable PGA output to EADC
  * @param[in]  pga     The pointer of the specified PGA module
  * @return     None
  * @details    This macro disable PGA output to EADC.
  */
#define PGA_DISABLE_OUTPUT_EADC(pga)    ((pga)->CTL &= ~PGA_CTL_PGACMPEN_Msk)


void PGA_Open(PGA_T *pga, uint32_t u32Gain, uint32_t u32OutputMask);
void PGA_Close(PGA_T *pga);

/*@}*/ /* end of group NM1120_PGA_EXPORTED_FUNCTIONS */

/*@}*/ /* end of group NM1120_PGA_Driver */

/*@}*/ /* end of group NM1120_Device_Driver */

#ifdef __cplusplus
}
#endif

#endif //__PGA_H__

/*** (C) COPYRIGHT 2016 Nuvoton Technology Corp. ***/
