/****************************************************************************//**
 * @file     usci_i2c.h
 * @version  V1.00
 * $Revision: 6 $
 * $Date: 16/03/30 9:16p $
 * @brief    NM1120 USCI_I2C driver header file
 *
 * @note
 * Copyright (C) 2016 Nuvoton Technology Corp. All rights reserved.
*****************************************************************************/
#ifndef __USCI_I2C_H__
#define __USCI_I2C_H__

#ifdef __cplusplus
extern "C"
{
#endif


/** @addtogroup NM1120_Device_Driver NM1120 Device Driver
  @{
*/

/** @addtogroup NM1120_USCI_I2C_Driver USCI_I2C Driver
  @{
*/

/** @addtogroup NM1120_USCI_I2C_EXPORTED_CONSTANTS USCI_I2C Exported Constants
  @{
*/

/*---------------------------------------------------------------------------------------------------------*/
/* USCI_I2C master event definitions                                                                            */
/*---------------------------------------------------------------------------------------------------------*/
enum USCI_I2C_MASTER_EVENT { 
				 MASTER_SEND_ADDRESS = 10,		/*!< Master send address to Slave */
                             MASTER_SEND_H_WR_ADDRESS,		/*!< Master send High address to Slave */
                             MASTER_SEND_H_RD_ADDRESS,	    	/*!< Master send address to Slave (Read ADDR) */
                             MASTER_SEND_L_ADDRESS,			/*!< Master send Low address to Slave */
                             MASTER_SEND_DATA,				/*!< Master Send Data to Slave */
                             MASTER_SEND_REPEAT_START,		/*!< Master send repeat start to Slave */
	                         MASTER_READ_DATA,				/*!< Master Get Data from Slave */
	                         MASTER_STOP,					/*!< Master send stop to Slave */
	                         MASTER_SEND_START				/*!< Master send start to Slave */
                           };

/*---------------------------------------------------------------------------------------------------------*/
/* USCI_I2C slave event definitions                                                                            */
/*---------------------------------------------------------------------------------------------------------*/
enum USCI_I2C_SLAVE_EVENT {
	                         SLAVE_ADDRESS_ACK = 100,                   /*!< Slave send address ACK */
	                         SLAVE_H_WR_ADDRESS_ACK,                 /*!< Slave send High address ACK */
	                         SLAVE_L_WR_ADDRESS_ACK,                 /*!< Slave send Low address ACK */
	                         SLAVE_GET_DATA,                      /*!< Slave Get Data from Master (Write CMD) */
	                         SLAVE_SEND_DATA,                      /*!< Slave Send Data to Master (Read CMD) */ 
                             SLAVE_H_RD_ADDRESS_ACK,		/*!< Slave send High address ACK */
	                         SLAVE_L_RD_ADDRESS_ACK 	/*!< Slave send Low address ACK */				
                          };

/*---------------------------------------------------------------------------------------------------------*/
/*  USCI_CTL constant definitions.                                                                            */
/*---------------------------------------------------------------------------------------------------------*/
#define USCI_I2C_CTL_PTRG               0x20UL /*!< USCI_CTL setting for I2C control bits. It would set PTRG bit                  */
#define USCI_I2C_CTL_STA               0x08UL /*!< USCI_CTL setting for I2C control bits. It would set STA bit                  */
#define USCI_I2C_CTL_STO               0x04UL /*!< USCI_CTL setting for I2C control bits. It would set STO bit                  */
#define USCI_I2C_CTL_AA                0x02UL /*!< USCI_CTL setting for I2C control bits. It would set AA bit                   */

/*---------------------------------------------------------------------------------------------------------*/
/*  USCI_I2C GCMode constant definitions.                                                                       */
/*---------------------------------------------------------------------------------------------------------*/
#define USCI_I2C_GCMODE_ENABLE           1    /*!< Enable  USCI_I2C GC Mode                                                         */
#define USCI_I2C_GCMODE_DISABLE          0    /*!< Disable USCI_I2C GC Mode                                                         */

/*---------------------------------------------------------------------------------------------------------*/
/*  USCI_I2C Wakeup Mode constant definitions.                                                                       */
/*---------------------------------------------------------------------------------------------------------*/
#define USCI_I2C_DATA_TOGGLE_WK      (0x0 << USCI_WKCTL_WKADDREN_Pos)    /*!< Wakeup according data toggle       */
#define USCI_I2C_ADDR_MATCH_WK       (0x1 << USCI_WKCTL_WKADDREN_Pos)    /*!< Wakeup according address match   */

/*---------------------------------------------------------------------------------------------------------*/
/* USCI_I2C interrupt mask definitions                                                                            */
/*---------------------------------------------------------------------------------------------------------*/
#define USCI_I2C_TO_INT_MASK                (0x001)                          /*!< Time-out interrupt mask */
#define USCI_I2C_STAR_INT_MASK               (0x002)                          /*!< Start condition received interrupt mask */
#define USCI_I2C_STOR_INT_MASK             (0x004)                          /*!< Stop condition received interrupt mask */
#define USCI_I2C_NACK_INT_MASK               (0x008)                          /*!< Non-acknowledge interrupt mask */
#define USCI_I2C_ARBLO_INT_MASK               (0x010)                          /*!< Arbitration lost interrupt mask */
#define USCI_I2C_ERR_INT_MASK               (0x020)                          /*!< Error interrupt mask */
#define USCI_I2C_ACK_INT_MASK                (0x040)                          /*!< Acknowledge interrupt mask */

/*@}*/ /* end of group NM1120_USCI_I2C_EXPORTED_CONSTANTS */


/** @addtogroup NM1120_USCI_I2C_EXPORTED_FUNCTIONS USCI_I2C Exported Functions
  @{
*/

/**
  * @brief This macro sets the USCI_I2C control register at one time.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @param[in] u8Ctrl The register value of USCI_I2C control register.
  * @return None
  * \hideinitializer
  */
#define USCI_I2C_SET_CONTROL_REG(usci, u8Ctrl) ((usci)->PROTCTL = ((usci)->PROTCTL & ~0x2E) | u8Ctrl)

/**
  * @brief This macro only set START bit to the control register of USCI_I2C module.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @return None
  * \hideinitializer
  */
#define USCI_I2C_START(usci) ( (usci)->PROTCTL = ((usci)->PROTCTL & ~USCI_PROTCTL_PTRG_Msk) | USCI_PROTCTL_STA_Msk)

/**
  * @brief This macro only set STOP bit to the control register of USCI_I2C module.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @return None
  * \hideinitializer
  */
#define USCI_I2C_STOP(usci) ((usci)->PROTCTL = ((usci)->PROTCTL & ~0x2E) | USCI_PROTCTL_STO_Msk)

/**
  * @brief This macro returns the data stored in data register of USCI_I2C module.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @return Data.
  * \hideinitializer
  */
#define USCI_I2C_GET_DATA(usci) ((usci)->RXDAT)

/**
  * @brief This macro writes the data to data register of USCI_I2C module.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @param[in] u8Data The data which will be written to data register of USCI_I2C module.
  * @return None
  * \hideinitializer
  */
#define USCI_I2C_SET_DATA(usci, u8Data) ((usci)->TXDAT = u8Data)

/**
  * @brief This macro returns time-out flag.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @return Status.
  * @retval 0 Flag is not set.
  * @retval 1 Flag is set.
  * \hideinitializer
  */
#define USCI_I2C_GET_TIMEOUT_FLAG(usci) (((usci)->PROTSTS & USCI_PROTSTS_TOIF_Msk) == USCI_PROTSTS_TOIF_Msk ? 1:0)

/**
  * @brief This macro returns wakeup flag.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @return Status.
  * @retval 0 Flag is not set.
  * @retval 1 Flag is set.
  * \hideinitializer
  */
#define USCI_I2C_GET_WAKEUP_FLAG(usci) (((usci)->WKSTS & USCI_WKSTS_WKF_Msk) == USCI_WKSTS_WKF_Msk ? 1:0)

/**
  * @brief This macro clears wakeup flag.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @return None
  * \hideinitializer  
  */
#define USCI_I2C_CLEAR_WAKEUP_FLAG(usci)  ((usci)->WKSTS |= USCI_WKSTS_WKF_Msk)

/**
  * @brief This macro disables the address 10 bit mode.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @return None
  * \hideinitializer  
  */
#define USCI_I2C_DISABLE_10BIT_ADDR_MODE(usci)  ((usci)->PROTCTL &= ~(USCI_PROTCTL_ADDR10EN_Msk))

/**
  * @brief This macro enables the address 10 bit mode.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @return None
  * \hideinitializer  
  */
#define USCI_I2C_ENABLE_10BIT_ADDR_MODE(usci)  ((usci)->PROTCTL |= USCI_PROTCTL_STO_Msk)

/**
  * @brief This macro gets protocol interrupt flag/status.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @return The interrupt flag/status of protocol status register.
  * \hideinitializer
  */
#define USCI_I2C_GET_PROT_STATUS(usci)    ((usci)->PROTSTS)

/**
  * @brief This macro clears specified protocol interrupt flag.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @param[in] u32IntTypeFlag Interrupt Type Flag, should be
  *                                  - \ref USCI_PROTSTS_ACKIF_Msk
  *                                  - \ref USCI_PROTSTS_ERRIF_Msk
  *                                  - \ref USCI_PROTSTS_ARBLOIF_Msk
  *                                  - \ref USCI_PROTSTS_NACKIF_Msk
  *                                  - \ref USCI_PROTSTS_STORIF_Msk
  *                                  - \ref USCI_PROTSTS_STARIF_Msk
  *                                  - \ref USCI_PROTSTS_TOIF_Msk
  * @return None
  * \hideinitializer
  */
#define USCI_I2C_CLR_PROT_INT_FLAG(usci,u32IntTypeFlag)    ((usci)->PROTSTS = u32IntTypeFlag)

/**
  * @brief This macro enables specified protocol interrupt.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @param[in] u32IntSel Interrupt Type, should be
  *                                  - \ref USCI_PROTIEN_ACKIEN_Msk
  *                                  - \ref USCI_PROTIEN_ERRIEN_Msk
  *                                  - \ref USCI_PROTIEN_ARBLOIEN_Msk  
  *                                  - \ref USCI_PROTIEN_NACKIEN_Msk
  *                                  - \ref USCI_PROTIEN_STORIEN_Msk
  *                                  - \ref USCI_PROTIEN_STARIEN_Msk
  *                                  - \ref USCI_PROTIEN_TOIEN_Msk
  * @return None
  * \hideinitializer
  */
#define USCI_I2C_ENABLE_PROT_INT(usci, u32IntSel)    ((usci)->PROTIEN |= (u32IntSel))

/**
  * @brief This macro disables specified protocol interrupt.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @param[in] u32IntSel Interrupt Type, should be
  *                                  - \ref USCI_PROTIEN_ACKIEN_Msk
  *                                  - \ref USCI_PROTIEN_ERRIEN_Msk
  *                                  - \ref USCI_PROTIEN_ARBLOIEN_Msk  
  *                                  - \ref USCI_PROTIEN_NACKIEN_Msk
  *                                  - \ref USCI_PROTIEN_STORIEN_Msk
  *                                  - \ref USCI_PROTIEN_STARIEN_Msk
  *                                  - \ref USCI_PROTIEN_TOIEN_Msk
  * @return None
  * \hideinitializer
  */
#define USCI_I2C_DISABLE_PROT_INT(usci, u32IntSel)    ((usci)->PROTIEN &= ~ (u32IntSel))


uint32_t USCI_I2C_Open(USCI_T *usci, uint32_t u32BusClock);
void USCI_I2C_Close(USCI_T *usci);
void USCI_I2C_ClearTimeoutFlag(USCI_T *usci);
void USCI_I2C_Trigger(USCI_T *usci, uint8_t u8Start, uint8_t u8Stop, uint8_t u8Ptrg, uint8_t u8Ack);
void USCI_I2C_DisableInt(USCI_T *usci, uint32_t u32Mask);
void USCI_I2C_EnableInt(USCI_T *usci, uint32_t u32Mask);
uint32_t USCI_I2C_GetBusClockFreq(USCI_T *usci);
uint32_t USCI_I2C_SetBusClockFreq(USCI_T *usci, uint32_t u32BusClock);
uint32_t USCI_I2C_GetIntFlag(USCI_T *usci, uint32_t u32Mask);
void USCI_I2C_ClearIntFlag(USCI_T* usci , uint32_t u32Mask);
uint32_t USCI_I2C_GetData(USCI_T *usci);
void USCI_I2C_SetData(USCI_T *usci, uint8_t u8Data);
void USCI_I2C_SetSlaveAddr(USCI_T *usci, uint8_t u8SlaveNo, uint16_t u16SlaveAddr, uint8_t u8GCMode);
void USCI_I2C_SetSlaveAddrMask(USCI_T *usci, uint8_t u8SlaveNo, uint16_t u16SlaveAddrMask);
void USCI_I2C_EnableTimeout(USCI_T *usci, uint32_t u32TimeoutCnt);
void USCI_I2C_DisableTimeout(USCI_T *usci);
void USCI_I2C_EnableWakeup(USCI_T *usci, uint8_t u8WakeupMode);
void USCI_I2C_DisableWakeup(USCI_T *usci);

/*@}*/ /* end of group NM1120_USCI_I2C_EXPORTED_FUNCTIONS */

/*@}*/ /* end of group NM1120_USCI_I2C_Driver */

/*@}*/ /* end of group NM1120_Device_Driver */

#ifdef __cplusplus
}
#endif

#endif //__USCI_I2C_H__

/*** (C) COPYRIGHT 2016 Nuvoton Technology Corp. ***/
