/****************************************************************************//**
 * @file     usci_spi.h
 * @version  V1.00
 * $Revision: 5 $
 * $Date: 16/03/30 10:06a $
 * @brief    NM1120 USCI_SPI driver header file
 *
 * @note
 * Copyright (C) 2016 Nuvoton Technology Corp. All rights reserved.
*****************************************************************************/
#ifndef __USCI_SPI_H__
#define __USCI_SPI_H__

#ifdef __cplusplus
extern "C"
{
#endif


/** @addtogroup NM1120_Device_Driver NM1120 Device Driver
  @{
*/

/** @addtogroup NM1120_USCI_SPI_Driver USCI_SPI Driver
  @{
*/

/** @addtogroup NM1120_USCI_SPI_EXPORTED_CONSTANTS USCI_SPI Exported Constants
  @{
*/

#define USCI_SPI_MODE_0        (0x0 << USCI_PROTCTL_SCLKMODE_Pos)        /*!< SCLK idle low; data transmit with falling edge and receive with rising edge */
#define USCI_SPI_MODE_1        (0x1 << USCI_PROTCTL_SCLKMODE_Pos)        /*!< SCLK idle low; data transmit with rising edge and receive with falling edge */
#define USCI_SPI_MODE_2        (0x2 << USCI_PROTCTL_SCLKMODE_Pos)        /*!< SCLK idle high; data transmit with rising edge and receive with falling edge */
#define USCI_SPI_MODE_3        (0x3 << USCI_PROTCTL_SCLKMODE_Pos)        /*!< SCLK idle high; data transmit with falling edge and receive with rising edge */

#define USCI_SPI_SLAVE         (USCI_PROTCTL_SLAVE_Msk)                             /*!< Set as slave */
#define USCI_SPI_MASTER        (0x0)                                           /*!< Set as master */

#define USCI_SPI_SS               (USCI_PROTCTL_SS_Msk)                                       /*!< Set SS */
#define USCI_SPI_SS_ACTIVE_HIGH    (0x0)                    /*!< SS active high */
#define USCI_SPI_SS_ACTIVE_LOW     (USCI_CTLIN0_ININV_Msk)                                       /*!< SS active low */

/* USCI_SPI Interrupt Mask */
#define USCI_SPI_SSINAIEN_MASK               (0x001)                        /*!< Slave Slave Inactive interrupt enable mask */
#define USCI_SPI_SSACTIEN_MASK               (0x002)                        /*!< Slave Slave Active interrupt enable mask */
#define USCI_SPI_SLVTOIEN_MASK               (0x004)                        /*!< Slave Mode Time-out interrupt enable mask */
#define USCI_SPI_SLVBEIEN_MASK               (0x008)                        /*!< Slave Mode Bit Count Error interrupt enable mask */
#define USCI_SPI_TXUDRIEN_MASK               (0x010)                        /*!< Slave Transmit Under Run interrupt enable mask */
#define USCI_SPI_RXOVIEN_MASK           (0x020)                        /*!< Receive Buffer Overrun interrupt enable mask */
#define USCI_SPI_TXSTIEN_MASK           (0x040)                        /*!< Transmit Start interrupt enable mask */
#define USCI_SPI_TXENDIEN_MASK           (0x080)                        /*!< Transmit End interrupt enable mask */
#define USCI_SPI_RXSTIEN_MASK           (0x100)                        /*!< Receive Start interrupt enable mask */
#define USCI_SPI_RXENDIEN_MASK           (0x200)                        /*!< Receive End interrupt enable mask */

/* USCI_SPI Status Mask */
#define SPI_BUSY_MASK                    (0x01)                           /*!< Busy status mask */
#define SPI_RX_EMPTY_MASK                (0x02)                           /*!< RX empty status mask */
#define SPI_RX_FULL_MASK                 (0x04)                           /*!< RX full status mask */
#define SPI_TX_EMPTY_MASK                (0x08)                           /*!< TX empty status mask */
#define SPI_TX_FULL_MASK                 (0x10)                           /*!< TX full status mask */
#define SPI_SSLINE_STS_MASK              (0x20)                           /*!< USCI_SPI_SS line status mask */

/*@}*/ /* end of group NM1120_USCI_SPI_EXPORTED_CONSTANTS */


/** @addtogroup NM1120_USCI_SPI_EXPORTED_FUNCTIONS USCI_SPI Exported Functions
  @{
*/

/**
  * @brief  Disable slave 3-wire mode.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @return None
  * \hideinitializer
  */
#define USCI_SPI_DISABLE_3WIRE_MODE(usci) ( (usci)->PROTCTL &= ~USCI_PROTCTL_SLV3WIRE_Msk )

/**
  * @brief  Enable slave 3-wire mode.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @return None
  * \hideinitializer
  */
#define USCI_SPI_ENABLE_3WIRE_MODE(usci) ( (usci)->PROTCTL |= USCI_PROTCTL_SLV3WIRE_Msk )

/**
  * @brief  Get the Rx buffer empty flag.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @return Rx buffer flag
  * @retval 0: Rx buffer is not empty
  * @retval 1: Rx buffer is empty
  * \hideinitializer
  */
#define USCI_SPI_GET_RX_EMPTY_FLAG(usci) ( ((usci)->BUFSTS & USCI_BUFSTS_RXEMPTY_Msk) == USCI_BUFSTS_RXEMPTY_Msk ? 1:0 )

/**
  * @brief  Get the Tx buffer empty flag.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @return Tx buffer flag
  * @retval 0: Tx buffer is not empty
  * @retval 1: Tx buffer is empty
  * \hideinitializer
  */
#define USCI_SPI_GET_TX_EMPTY_FLAG(usci) ( ((usci)->BUFSTS & USCI_BUFSTS_TXEMPTY_Msk) == USCI_BUFSTS_TXEMPTY_Msk ? 1:0 )

/**
  * @brief  Get the Tx buffer full flag.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @return Tx buffer flag
  * @retval 0: Tx buffer is not full
  * @retval 1: Tx buffer is full
  * \hideinitializer
  */
#define USCI_SPI_GET_TX_FULL_FLAG(usci) ( ((usci)->BUFSTS & USCI_BUFSTS_TXFULL_Msk) == USCI_BUFSTS_TXFULL_Msk ? 1:0 )

/**
  * @brief  Get the datum read from RX register.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @return data in Rx register
  * \hideinitializer
  */
#define USCI_SPI_READ_RX(usci) ((usci)->RXDAT)

/**
  * @brief  Write datum to TX register.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @param[in]  u32TxData The datum which user attempt to transfer through USCI_SPI bus.
  * @return None
  * \hideinitializer
  */
#define USCI_SPI_WRITE_TX(usci, u32TxData) ( (usci)->TXDAT = u32TxData )

/**
  * @brief      Set USCI_SPI_SS pin to high state.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @return     None.
  * @details    Disable automatic slave selection function and set USCI_SPI_SS pin to high state. Only available in Master mode.
  * \hideinitializer
  */
#define USCI_SPI_SET_SS_HIGH(usci) \
    do{ \
        (usci)->CTLIN0 &= ~(USCI_CTLIN0_ININV_Msk); \
        (usci)->PROTCTL = ((usci)->PROTCTL & ~(USCI_PROTCTL_AUTOSS_Msk)) | USCI_PROTCTL_SS_Msk); \
    }while(0)
    
/**
  * @brief      Set USCI_SPI_SS pin to low state.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @return     None.
  * @details    Disable automatic slave selection function and set USCI_SPI_SS pin to low state. Only available in Master mode.
  * \hideinitializer
  */
#define USCI_SPI_SET_SS_LOW(usci) \
    do{ \
        (usci)->CTLIN0 |= (USCI_CTLIN0_ININV_Msk); \
        (usci)->PROTCTL = ((usci)->PROTCTL & ~USCI_PROTCTL_AUTOSS_Msk) | USCI_PROTCTL_SS_Msk); \
    }while(0)

/**
  * @brief  Set the length of suspend interval.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @param[in]  u32SuspCycle Decide the length of suspend interval.
  * @return None
  * \hideinitializer
  */
#define USCI_SPI_SET_SUSPEND_CYCLE(usci, u32SuspCycle) ( (usci)->PROTCTL = ((usci)->PROTCTL & ~USCI_PROTCTL_SUSPITV_Msk) | (u32SuspCycle << USCI_PROTCTL_SUSPITV_Pos) )

/**
  * @brief  Set the USCI_SPI transfer sequence with LSB first.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @return None
  * \hideinitializer
  */
#define USCI_SPI_SET_LSB_FIRST(usci) ( (usci)->LINECTL |= USCI_LINECTL_LSB_Msk )

/**
  * @brief  Set the USCI_SPI transfer sequence with MSB first.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @return None
  * \hideinitializer
  */
#define USCI_SPI_SET_MSB_FIRST(usci) ( (usci)->LINECTL &= ~USCI_LINECTL_LSB_Msk )

/**
  * @brief  Set the data width of a USCI_SPI transaction.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @param[in]  u32Width The data width
  * @return None
  * \hideinitializer
  */
static __INLINE void USCI_SPI_SET_DATA_WIDTH(USCI_T *usci, uint32_t u32Width)
{
    if(u32Width == 16)
        u32Width = 0;

    usci->LINECTL = (usci->LINECTL & ~USCI_LINECTL_DWIDTH_Msk) | (u32Width << USCI_LINECTL_DWIDTH_Pos);
}

/**
  * @brief  Get the USCI_SPI busy state.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @return USCI_SPI busy status
  * @retval 0: USCI_SPI module is not busy
  * @retval 1: USCI_SPI module is busy
  * \hideinitializer
  */
#define USCI_SPI_IS_BUSY(usci) ( ((usci)->PROTSTS & USCI_PROTSTS_BUSY_Msk) == USCI_PROTSTS_BUSY_Msk ? 1:0 )

/**
  * @brief Get the USCI_SPI wakeup flag.
  * @param[in] usci The pointer of the specified USCI_SPI module.
  * @return Wakeup status.
  * @retval 0 Flag is not set.
  * @retval 1 Flag is set.
  * \hideinitializer
  */
#define USCI_SPI_GET_WAKEUP_FLAG(usci) ( ((usci)->WKSTS & USCI_WKSTS_WKF_Msk) == USCI_WKSTS_WKF_Msk ? 1:0)

/**
  * @brief Clear the USCI_SPI wakeup flag.
  * @param[in] usci The pointer of the specified USCI_SPI module.
  * @return None
  * \hideinitializer  
  */
#define USCI_SPI_CLEAR_WAKEUP_FLAG(usci)  ( (usci)->WKSTS |= USCI_WKSTS_WKF_Msk)

/**
  * @brief Get protocol interrupt flag/status.
  * @param[in] usci The pointer of the specified USCI_SPI module.
  * @return The interrupt flag/status of protocol status register.
  * \hideinitializer
  */
#define USCI_SPI_GET_PROT_STATUS(usci)    ( (usci)->PROTSTS)

/**
  * @brief Clear specified protocol interrupt flag.
  * @param[in] usci The pointer of the specified USCI_SPI module.
  * @param[in] u32IntTypeFlag Interrupt Type Flag, should be
  *                                  - \ref USCI_PROTSTS_SSACTIF_Msk
  *                                  - \ref USCI_PROTSTS_SSINAIF_Msk
  *                                  - \ref USCI_PROTSTS_SLVBEIF_Msk
  *                                  - \ref USCI_PROTSTS_SLVTOIF_Msk
  *                                  - \ref USCI_PROTSTS_RXENDIF_Msk
  *                                  - \ref USCI_PROTSTS_RXSTIF_Msk
  *                                  - \ref USCI_PROTSTS_TXENDIF_Msk
  *                                  - \ref USCI_PROTSTS_TXSTIF_Msk  
  * @return None
  * \hideinitializer
  */
#define USCI_SPI_CLR_PROT_INT_FLAG(usci,u32IntTypeFlag)    ( (usci)->PROTSTS = u32IntTypeFlag)

/**
  * @brief Get buffer interrupt flag/status.
  * @param[in] usci The pointer of the specified USCI_SPI module.
  * @return The interrupt flag/status of buffer status register.
  * \hideinitializer
  */
#define USCI_SPI_GET_BUF_STATUS(usci)    ( (usci)->BUFSTS)

/**
  * @brief Clear specified buffer interrupt flag.
  * @param[in] usci The pointer of the specified USCI_SPI module.
  * @param[in] u32IntTypeFlag Interrupt Type Flag, should be
  *                                  - \ref USCI_BUFSTS_TXUDRIF_Msk
  *                                  - \ref USCI_BUFSTS_RXOVIF_Msk
  * @return None
  * \hideinitializer
  */
#define USCI_SPI_CLR_BUF_INT_FLAG(usci,u32IntTypeFlag)    ( (usci)->BUFSTS = u32IntTypeFlag)

/**
  * @brief Enable specified protocol interrupt.
  * @param[in] usci The pointer of the specified USCI_SPI module.
  * @param[in] u32IntSel Interrupt Type, should be
  *                                  - \ref USCI_PROTIEN_SLVBEIEN_Msk
  *                                  - \ref USCI_PROTIEN_SLVTOIEN_Msk
  *                                  - \ref USCI_PROTIEN_SSACTIEN_Msk  
  *                                  - \ref USCI_PROTIEN_SSINAIEN_Msk
  * @return None
  * \hideinitializer
  */
#define USCI_SPI_ENABLE_PROT_INT(usci, u32IntSel)    ((usci)->PROTIEN |= (u32IntSel))

/**
  * @brief Disable specified protocol interrupt.
  * @param[in] usci The pointer of the specified USCI_SPI module.
  * @param[in] u32IntSel Interrupt Type, should be
  *                                  - \ref USCI_PROTIEN_SLVBEIEN_Msk
  *                                  - \ref USCI_PROTIEN_SLVTOIEN_Msk
  *                                  - \ref USCI_PROTIEN_SSACTIEN_Msk  
  *                                  - \ref USCI_PROTIEN_SSINAIEN_Msk
  * @return None
  * \hideinitializer
  */
#define USCI_SPI_DISABLE_PROT_INT(usci, u32IntSel)    ((usci)->PROTIEN &= ~ (u32IntSel))

/**
  * @brief Enable specified buffer interrupt.
  * @param[in] usci The pointer of the specified USCI_SPI module.
  * @param[in] u32IntSel Interrupt Type, should be
  *                                  - \ref USCI_BUFCTL_RXOVIEN_Msk
  *                                  - \ref USCI_BUFCTL_TXUDRIEN_Msk
  * @return None
  * \hideinitializer
  */
#define USCI_SPI_ENABLE_BUF_INT(usci, u32IntSel)    ((usci)->BUFCTL |= (u32IntSel))

/**
  * @brief Disable specified buffer interrupt.
  * @param[in] usci The pointer of the specified USCI_SPI module.
  * @param[in] u32IntSel Interrupt Type, should be
  *                                  - \ref USCI_BUFCTL_RXOVIEN_Msk
  *                                  - \ref USCI_BUFCTL_TXUDRIEN_Msk
  * @return None
  * \hideinitializer
  */
#define USCI_SPI_DISABLE_BUF_INT(usci, u32IntSel)    ((usci)->BUFCTL &= ~ (u32IntSel))

/**
  * @brief Enable specified transfer interrupt.
  * @param[in] usci The pointer of the specified USCI_SPI module.
  * @param[in] u32IntSel Interrupt Type, should be
  *                                  - \ref USCI_INTEN_RXENDIEN_Msk
  *                                  - \ref USCI_INTEN_RXSTIEN_Msk
  *                                  - \ref USCI_INTEN_TXENDIEN_Msk
  *                                  - \ref USCI_INTEN_TXSTIEN_Msk  
  * @return None
  * \hideinitializer
  */
#define USCI_SPI_ENABLE_TRANS_INT(usci, u32IntSel)    ((usci)->INTEN |= (u32IntSel))

/**
  * @brief Disable specified transfer interrupt.
  * @param[in] usci The pointer of the specified USCI_SPI module.
  * @param[in] u32IntSel Interrupt Type, should be
  *                                  - \ref USCI_INTEN_RXENDIEN_Msk
  *                                  - \ref USCI_INTEN_RXSTIEN_Msk
  *                                  - \ref USCI_INTEN_TXENDIEN_Msk
  *                                  - \ref USCI_INTEN_TXSTIEN_Msk
  * @return None
  * \hideinitializer
  */
#define USCI_SPI_DISABLE_TRANS_INT(usci, u32IntSel)    ((usci)->INTEN &= ~ (u32IntSel))

uint32_t USCI_SPI_Open(USCI_T *usci, uint32_t u32MasterSlave, uint32_t u32SPIMode,  uint32_t u32DataWidth, uint32_t u32BusClock);
void USCI_SPI_Close(USCI_T *usci);
void USCI_SPI_ClearRxBuf(USCI_T *usci);
void USCI_SPI_ClearTxBuf(USCI_T *usci);
void USCI_SPI_DisableAutoSS(USCI_T *usci);
void USCI_SPI_EnableAutoSS(USCI_T *usci, uint32_t u32SSPinMask, uint32_t u32ActiveLevel);
uint32_t USCI_SPI_SetBusClock(USCI_T *usci, uint32_t u32BusClock);
uint32_t USCI_SPI_GetBusClock(USCI_T *usci);
void USCI_SPI_EnableInt(USCI_T *usci, uint32_t u32Mask);
void USCI_SPI_DisableInt(USCI_T *usci, uint32_t u32Mask);
uint32_t USCI_SPI_GetIntFlag(USCI_T *usci, uint32_t u32Mask);
void USCI_SPI_ClearIntFlag(USCI_T *usci, uint32_t u32Mask);
uint32_t USCI_SPI_GetStatus(USCI_T *usci, uint32_t u32Mask);
void USCI_SPI_EnableWakeup(USCI_T *usci);
void USCI_SPI_DisableWakeup(USCI_T *usci);


/*@}*/ /* end of group NM1120_USCI_SPI_EXPORTED_FUNCTIONS */

/*@}*/ /* end of group NM1120_USCI_SPI_Driver */

/*@}*/ /* end of group NM1120_Device_Driver */

#ifdef __cplusplus
}
#endif

#endif //__USCI_SPI_H__

/*** (C) COPYRIGHT 2016 Nuvoton Technology Corp. ***/

