/******************************************************************************
 * @file     usci_uart.h
 * @version  V3.00
 * $Revision: 9 $
 * $Date: 16/03/30 10:06a $
 * @brief    NM1120 USCI_UART driver header file
 *
 * @note
 * Copyright (C) 2016 Nuvoton Technology Corp. All rights reserved.
*****************************************************************************/
#ifndef __USCI_UART_H__
#define __USCI_UART_H__


#ifdef __cplusplus
extern "C"
{
#endif


/** @addtogroup NM1120_Device_Driver NM1120 Device Driver
  @{
*/

/** @addtogroup NM1120_USCI_UART_Driver USCI_UART Driver
  @{
*/

/** @addtogroup NM1120_USCI_UART_EXPORTED_CONSTANTS USCI_UART Exported Constants
  @{
*/

/*---------------------------------------------------------------------------------------------------------*/
/* USCI_LINECTL constants definitions                                                                            */
/*---------------------------------------------------------------------------------------------------------*/
#define USCI_UART_WORD_LEN_6     (6 << USCI_LINECTL_DWIDTH_Pos) /*!< USCI_LINECTL setting to set UART word length to 6 bits */
#define USCI_UART_WORD_LEN_7     (7 << USCI_LINECTL_DWIDTH_Pos) /*!< USCI_LINECTL setting to set UART word length to 7 bits */
#define USCI_UART_WORD_LEN_8     (8 << USCI_LINECTL_DWIDTH_Pos) /*!< USCI_LINECTL setting to set UART word length to 8 bits */

/*---------------------------------------------------------------------------------------------------------*/
/* USCI_PROTCTL constants definitions                                                                            */
/*---------------------------------------------------------------------------------------------------------*/
#define USCI_UART_PARITY_NONE    (0x0 << USCI_PROTCTL_PARITYEN_Pos) /*!< USCI_PROTCTL setting to set UART as no parity */
#define USCI_UART_PARITY_ODD     (0x1 << USCI_PROTCTL_PARITYEN_Pos) /*!< USCI_PROTCTL setting to set UART as odd parity */
#define USCI_UART_PARITY_EVEN    (0x3 << USCI_PROTCTL_PARITYEN_Pos) /*!< USCI_PROTCTL setting to set UART as even parity */
#define USCI_UART_PARITY_MARK    (0x2000001 << USCI_PROTCTL_PARITYEN_Pos) /*!< USCI_PROTCTL setting to keep parity bit as '1' */
#define USCI_UART_PARITY_SPACE   (0x2000003 << USCI_PROTCTL_PARITYEN_Pos) /*!< USCI_PROTCTL setting to keep parity bit as '0' */

#define USCI_UART_STOP_BIT_1     (0x0) /*!< USCI_PROTCTL setting for one stop bit */
#define USCI_UART_STOP_BIT_2     (0x1) /*!< USCI_PROTCTL setting for two stop bit */

/*---------------------------------------------------------------------------------------------------------*/
/* USCI UART interrupt mask definitions                                                                            */
/*---------------------------------------------------------------------------------------------------------*/
#define USCI_UART_BRK_INT_MASK                (0x001)                          /*!< LIN break detected interrupt mask */
#define USCI_UART_ABR_INT_MASK               (0x002)                          /*!< Auto-baud rate interrupt mask */
#define USCI_UART_RLS_INT_MASK             (0x004)                          /*!< Receive line status interrupt mask */
#define USCI_UART_BUF_RXOV_INT_MASK               (0x008)                          /*!< Buffer RX overrun interrupt mask */
#define USCI_UART_TXST_INT_MASK               (0x010)                          /*!< TX start interrupt mask */
#define USCI_UART_TXEND_INT_MASK               (0x020)                          /*!< Tx end interrupt mask */
#define USCI_UART_RXST_INT_MASK                (0x040)                          /*!< RX start interrupt mask */
#define USCI_UART_RXEND_INT_MASK           (0x080)                          /*!< RX end interrupt mask */


/*@}*/ /* end of group NM1120_USCI_UART_EXPORTED_CONSTANTS */


/** @addtogroup NM1120_USCI_UART_EXPORTED_FUNCTIONS USCI_UART Exported Functions
  @{
*/


/**
 *    @brief        Write USCI_UART data
 *
 *    @param[in]    usci    The pointer of the specified USCI_UART module
 *    @param[in]    u8Data  Data byte to transmit.
 *
 *    @return       None
 *
 *    @details      This macro write Data to Tx data register.
 */
#define USCI_UART_WRITE(usci, u8Data)    ((usci)->TXDAT = (u8Data))


/**
 *    @brief        Read USCI_UART data
 *
 *    @param[in]    usci    The pointer of the specified USCI_UART module
 *
 *    @return       The oldest data byte in RX buffer.
 *
 *    @details      This macro read Rx data register.
 */
#define USCI_UART_READ(usci)    ((usci)->RXDAT)


/**
 *    @brief        Get Tx empty
 *
 *    @param[in]    usci    The pointer of the specified USCI_UART module
 *
 *    @retval       0   Tx buffer is not empty
 *    @retval       >=1 Tx buffer is empty
 *
 *    @details      This macro get Transmitter buffer empty register value.
 */
#define USCI_UART_GET_TX_EMPTY(usci)    ((usci)->BUFSTS & USCI_BUFSTS_TXEMPTY_Msk)


/**
 *    @brief        Get Rx empty
 *
 *    @param[in]    usci    The pointer of the specified USCI_UART module
 *
 *    @retval       0   Rx buffer is not empty
 *    @retval       >=1 Rx buffer is empty
 *
 *    @details      This macro get Receiver buffer empty register value.
 */
#define USCI_UART_GET_RX_EMPTY(usci)    ((usci)->BUFSTS & USCI_BUFSTS_RXEMPTY_Msk)


/**
 *    @brief        Check specified usci_uart port transmission is over.
 *
 *    @param[in]    usci    The pointer of the specified USCI_UART module
 *
 *    @retval       0 Tx transmission is not over
 *    @retval       1 Tx transmission is over
 *
 *    @details      This macro return Transmitter Empty Flag register bit value.
 *                  It indicates if specified usci_uart port transmission is over nor not.
 */
#define USCI_UART_IS_TX_EMPTY(usci)    (((usci)->BUFSTS & USCI_BUFSTS_TXEMPTY_Msk) >> USCI_BUFSTS_TXEMPTY_Pos)


/**
 *    @brief        Check specified usci_uart port receiver is empty.
 *
 *    @param[in]    usci    The pointer of the specified USCI_UART module
 *
 *    @retval       0 Rx receiver is not empty
 *    @retval       1 Rx receiver is empty
 *
 *    @details      This macro return Receive Empty Flag register bit value.
 *                  It indicates if specified usci_uart port receiver is empty nor not.
 */
#define USCI_UART_IS_RX_EMPTY(usci)    (((usci)->BUFSTS & USCI_BUFSTS_RXEMPTY_Msk) >> USCI_BUFSTS_RXEMPTY_Pos)


/**
 *    @brief        Wait specified usci_uart port transmission is over
 *
 *    @param[in]    usci    The pointer of the specified USCI_UART module
 *
 *    @return       None
 *
 *    @details      This macro wait specified usci_uart port transmission is over.
 */
#define USCI_UART_WAIT_TX_EMPTY(usci)    while(!((((usci)->BUFSTS) & USCI_BUFSTS_TXEMPTY_Msk) >> USCI_BUFSTS_TXEMPTY_Pos))


/**
 *    @brief        Check TX buffer is full or not
 *
 *    @param[in]    usci    The pointer of the specified USCI_UART module
 *
 *    @retval       1 TX buffer is full
 *    @retval       0 TX buffer is not full
 *
 *    @details      This macro check TX buffer is full or not.
 */
#define USCI_UART_IS_TX_FULL(usci)    (((usci)->BUFSTS & USCI_BUFSTS_TXFULL_Msk)>>USCI_BUFSTS_TXFULL_Pos)


/**
 *    @brief        Check RX buffer is full or not
 *
 *    @param[in]    usci    The pointer of the specified USCI_UART module
 *
 *    @retval       1 RX buffer is full
 *    @retval       0 RX buffer is not full
 *
 *    @details      This macro check RX buffer is full or not.
 */
#define USCI_UART_IS_RX_FULL(usci)    (((usci)->BUFSTS & USCI_BUFSTS_RXFULL_Msk)>>USCI_BUFSTS_RXFULL_Pos)


/**
 *    @brief        Get Tx full register value
 *
 *    @param[in]    usci    The pointer of the specified USCI_UART module
 *
 *    @retval       0   Tx buffer is not full.
 *    @retval       >=1 Tx buffer is full.
 *
 *    @details      This macro get Tx full register value.
 */
#define USCI_UART_GET_TX_FULL(usci)    ((usci)->BUFSTS & USCI_BUFSTS_TXFULL_Msk)


/**
 *    @brief        Get Rx full register value
 *
 *    @param[in]    usci    The pointer of the specified USCI_UART module
 *
 *    @retval       0   Rx buffer is not full.
 *    @retval       >=1 Rx buffer is full.
 *
 *    @details      This macro get Rx full register value.
 */
#define USCI_UART_GET_RX_FULL(usci)    ((usci)->BUFSTS & USCI_BUFSTS_RXFULL_Msk)


/**
 *    @brief        Enable specified USCI_UART protocol interrupt
 *
 *    @param[in]    usci        The pointer of the specified USCI_UART module
 *    @param[in]    u32IntSel  Interrupt type select
 *                              - \ref USCI_PROTIEN_RLSIEN_Msk     : Rx Line status interrupt
 *                              - \ref USCI_PROTIEN_ABRIEN_Msk   : Auto-baud rate interrupt
 *                              - \ref USCI_PROTIEN_BRKIEN_Msk   : LIN break detected interrupt
 *
 *    @return       None
 *
 *    @details      This macro enable specified USCI_UART protocol interrupt.
 */
#define USCI_UART_ENABLE_PROT_INT(usci, u32IntSel)    ((usci)->PROTIEN |= (u32IntSel))


/**
 *    @brief        Disable specified USCI_UART protocol interrupt
 *
 *    @param[in]    usci        The pointer of the specified USCI_UART module
 *    @param[in]    u32IntSel  Interrupt type select
 *                              - \ref USCI_PROTIEN_RLSIEN_Msk     : Rx Line status interrupt
 *                              - \ref USCI_PROTIEN_ABRIEN_Msk   : Auto-baud rate interrupt
 *                              - \ref USCI_PROTIEN_BRKIEN_Msk   : LIN break detected interrupt
 *
 *    @return       None
 *
 *    @details      This macro disable specified USCI_UART protocol interrupt.
 */
#define USCI_UART_DISABLE_PROT_INT(usci, u32IntSel)    ((usci)->PROTIEN &= ~ (u32IntSel))


/**
 *    @brief        Enable specified USCI_UART buffer interrupt
 *
 *    @param[in]    usci        The pointer of the specified USCI_UART module
 *    @param[in]    u32IntSel  Interrupt type select
 *                              - \ref USCI_BUFCTL_RXOVIEN_Msk     : Receive buffer overrun error interrupt
 *
 *    @return       None
 *
 *    @details      This macro enable specified USCI_UART buffer interrupt.
 */
#define USCI_UART_ENABLE_BUF_INT(usci, u32IntSel)    ((usci)->BUFCTL |= (u32IntSel))


/**
 *    @brief        Disable specified USCI_UART buffer interrupt
 *
 *    @param[in]    usci        The pointer of the specified USCI_UART module
 *    @param[in]    u32IntSel  Interrupt type select
 *                              - \ref USCI_BUFCTL_RXOVIEN_Msk     : Receive buffer overrun error interrupt
 *
 *    @return       None
 *
 *    @details      This macro disable specified USCI_UART buffer interrupt.
 */
#define USCI_UART_DISABLE_BUF_INT(usci, u32IntSel)    ((usci)->BUFCTL &= ~ (u32IntSel))


/**
 *    @brief        Enable specified USCI_UART transfer interrupt
 *
 *    @param[in]    usci        The pointer of the specified USCI_UART module
 *    @param[in]    u32IntSel  Interrupt type select
 *                              - \ref USCI_INTEN_RXENDIEN_Msk     : Receive end interrupt
 *                              - \ref USCI_INTEN_RXSTIEN_Msk   : Receive start interrupt
 *                              - \ref USCI_INTEN_TXENDIEN_Msk   : Transmit end interrupt
 *                              - \ref USCI_INTEN_TXSTIEN_Msk     : Transmit start interrupt
 *
 *    @return       None
 *
 *    @details      This macro enable specified USCI_UART transfer interrupt.
 */
#define USCI_UART_ENABLE_TRANS_INT(usci, u32IntSel)    ((usci)->INTEN |= (u32IntSel))


/**
 *    @brief        Disable specified USCI_UART transfer interrupt
 *
 *    @param[in]    usci        The pointer of the specified USCI_UART module
 *    @param[in]    u32IntSel  Interrupt type select
 *                              - \ref USCI_INTEN_RXENDIEN_Msk     : Receive end interrupt
 *                              - \ref USCI_INTEN_RXSTIEN_Msk   : Receive start interrupt
 *                              - \ref USCI_INTEN_TXENDIEN_Msk   : Transmit end interrupt
 *                              - \ref USCI_INTEN_TXSTIEN_Msk     : Transmit start interrupt
 *
 *    @return       None
 *
 *    @details      This macro disable specified USCI_UART transfer interrupt.
 */
#define USCI_UART_DISABLE_TRANS_INT(usci, u32IntSel)    ((usci)->INTEN &= ~ (u32IntSel))


/**
 *    @brief        Get protocol interrupt flag/status
 *
 *    @param[in]    usci        The pointer of the specified USCI_UART module
 *
 *    @return       The interrupt flag/status of protocol status register.
 *
 *    @details      This macro get protocol status register value.
 */
#define USCI_UART_GET_PROT_STATUS(usci)    ((usci)->PROTSTS)


/**
 *    @brief        Clear specified protocol interrupt flag
 *
 *    @param[in]    usci        The pointer of the specified USCI_UART module
 *    @param[in]    u32IntTypeFlag Interrupt Type Flag, should be
 *                                  - \ref USCI_PROTSTS_ABERRSTS_Msk : Auto-baud Rate Error Interrupt Indicator
 *                                  - \ref USCI_PROTSTS_ABRDETIF_Msk   : Auto-baud Rate Detected Interrupt Flag
 *                                  - \ref USCI_PROTSTS_BRKDETIF_Msk  : LIN Break Detected Interrupt Flag
 *                                  - \ref USCI_PROTSTS_BREAK_Msk  : Break Flag
 *                                  - \ref USCI_PROTSTS_FRMERR_Msk  : Framing Error Flag
 *                                  - \ref USCI_PROTSTS_PARITYERR_Msk    : Parity Error Flag
 *                                  - \ref USCI_PROTSTS_RXENDIF_Msk   : Receive End Interrupt Flag
 *                                  - \ref USCI_PROTSTS_RXSTIF_Msk   : Receive Start Interrupt Flag
 *                                  - \ref USCI_PROTSTS_TXENDIF_Msk   : Transmit End Interrupt Flag
 *                                  - \ref USCI_PROTSTS_TXSTIF_Msk   : Transmit Start Interrupt Flag
 *
 *    @return       None
 *
 *    @details      This macro clear specified protocol interrupt flag.
 */
#define USCI_UART_CLR_PROT_INT_FLAG(usci,u32IntTypeFlag)    ((usci)->PROTSTS = u32IntTypeFlag)


/**
 *    @brief        Get transmit/receive buffer interrupt flag/status
 *
 *    @param[in]    usci        The pointer of the specified USCI_UART module
 *
 *    @return       The interrupt flag/status of buffer status register.
 *
 *    @details      This macro get buffer status register value.
 */
#define USCI_UART_GET_BUF_STATUS(usci)    ((usci)->BUFSTS)


/**
 *    @brief        Clear specified buffer interrupt flag
 *
 *    @param[in]    usci        The pointer of the specified USCI_UART module
 *    @param[in]    u32IntTypeFlag Interrupt Type Flag, should be
 *                                  - \ref USCI_BUFSTS_RXOVIF_Msk : Receive Buffer Over-run Error  Interrupt Indicator
 *
 *    @return       None
 *
 *    @details      This macro clear specified buffer interrupt flag.
 */
#define USCI_UART_CLR_BUF_INT_FLAG(usci,u32IntTypeFlag)    ((usci)->BUFSTS = u32IntTypeFlag)


/**
 *    @brief        Get wakeup flag
 *
 *    @param[in]    usci        The pointer of the specified USCI_UART module
 *
 *    @retval       0       Chip did not wake up from power-down mode.
 *    @retval       1       Chip waked up from power-down mode.
 *
 *    @details      This macro get wakeup flag.
 */
#define USCI_UART_GET_WK_FLAG(usci)    ((usci)->WKSTS & USCI_WKSTS_WKF_Msk ? 1:0  )


/**
 *    @brief        Clear wakeup flag
 *
 *    @param[in]    usci        The pointer of the specified USCI_UART module
 *
 *    @return       None
 *
 *    @details      This macro clear wakeup flag.
 */
#define USCI_UART_CLR_WK_FLAG(usci)    ((usci)->WKSTS = USCI_WKSTS_WKF_Msk)


void USCI_UART_ClearIntFlag(USCI_T* usci , uint32_t u32Mask);
uint32_t USCI_UART_GetIntFlag(USCI_T* usci , uint32_t u32Mask);
void USCI_UART_Close(USCI_T* usci);
void USCI_UART_DisableInt(USCI_T*  usci, uint32_t u32Mask);
void USCI_UART_EnableInt(USCI_T*  usci, uint32_t u32Mask);
uint32_t USCI_UART_Open(USCI_T* usci, uint32_t u32baudrate);
uint32_t USCI_UART_Read(USCI_T* usci, uint8_t *pu8RxBuf, uint32_t u32ReadBytes);
uint32_t USCI_UART_SetLine_Config(USCI_T* usci, uint32_t u32baudrate, uint32_t u32data_width, uint32_t u32parity, uint32_t u32stop_bits);
void USCI_UART_SelectLINMode(USCI_T* usci, uint32_t u32Mode);
uint32_t USCI_UART_Write(USCI_T* usci, uint8_t *pu8TxBuf, uint32_t u32WriteBytes);
void USCI_UART_EnableWakeup(USCI_T* usci, uint32_t u32WakeupMode);
void USCI_UART_DisableWakeup(USCI_T* usci);


/*@}*/ /* end of group NM1120_USCI_UART_EXPORTED_FUNCTIONS */

/*@}*/ /* end of group NM1120_USCI_UART_Driver */

/*@}*/ /* end of group NM1120_Device_Driver */

#ifdef __cplusplus
}
#endif

#endif //__USCI_UART_H__

/*** (C) COPYRIGHT 2016 Nuvoton Technology Corp. ***/
