/****************************************************************************//**
 * @file     usci_i2c.c
 * @version  V0.10
 * $Revision: 5 $
 * $Date: 16/03/30 10:06a $
 * @brief    NM1120 USCI_I2C driver source file
 *
 * @note
 * Copyright (C) 2016 Nuvoton Technology Corp. All rights reserved.
*****************************************************************************/

#include "NM1120.h"

/** @addtogroup NM1120_Device_Driver NM1120 Device Driver
  @{
*/

/** @addtogroup NM1120_USCI_I2C_Driver USCI_I2C Driver
  @{
*/


/** @addtogroup NM1120_USCI_I2C_EXPORTED_FUNCTIONS USCI_I2C Exported Functions
  @{
*/

/**
  * @brief This function make USCI_I2C module be ready and set the wanted bus clock.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @param[in] u32BusClock The target bus speed of USCI_I2C module.
  * @return Actual USCI_I2C bus clock frequency.
  */
uint32_t USCI_I2C_Open(USCI_T *usci, uint32_t u32BusClock)
{
    uint32_t u32ClkDiv;
    uint32_t u32Pclk = CLK_GetPCLKFreq();

    u32ClkDiv = (uint32_t) ((((((u32Pclk/2)*10)/(u32BusClock))+5)/10)-1); /* Compute proper divider for USCI_I2C clock */

    /* Enable USCI_I2C protocol */
    usci->CTL &= ~USCI_CTL_FUNMODE_Msk;
    usci->CTL = 4 << USCI_CTL_FUNMODE_Pos;

    /* Data format configuration */
    /* 8 bit data length */
    usci->LINECTL &= ~USCI_LINECTL_DWIDTH_Msk;
    usci->LINECTL |= 8 << USCI_LINECTL_DWIDTH_Pos;

    /* MSB data format */
    usci->LINECTL &= ~USCI_LINECTL_LSB_Msk;

    /* Set USCI_I2C bus clock */
    usci->BRGEN &= ~USCI_BRGEN_CLKDIV_Msk;
    usci->BRGEN |=  (u32ClkDiv << USCI_BRGEN_CLKDIV_Pos);
    usci->PROTCTL |=  USCI_PROTCTL_PROTEN_Msk;

    return ( u32Pclk / ((u32ClkDiv+1)<<1) );
}

/**
  * @brief  This function closes the USCI_I2C module.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @return None
  */
void USCI_I2C_Close(USCI_T *usci)
{
    /* Disable USCI_I2C function */
    usci->CTL &= ~USCI_CTL_FUNMODE_Msk;
}

/**
  * @brief This function clears the time-out flag.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @return None
  */
void USCI_I2C_ClearTimeoutFlag(USCI_T *usci)
{
    usci->PROTSTS = USCI_PROTSTS_TOIF_Msk;
}

/**
  * @brief This function sets the control bit of the USCI_I2C module.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @param[in] u8Start Set START bit to USCI_I2C module.
  * @param[in] u8Stop Set STOP bit to USCI_I2C module.
  * @param[in] u8Ptrg Set PTRG bit to USCI_I2C module.
  * @param[in] u8Ack Set ACK bit to USCI_I2C module.
  * @return None
  */
void USCI_I2C_Trigger(USCI_T *usci, uint8_t u8Start, uint8_t u8Stop, uint8_t u8Ptrg, uint8_t u8Ack)
{
    uint32_t u32Reg = 0;
    uint32_t u32Val = usci->PROTCTL & ~(USCI_PROTCTL_STA_Msk | USCI_PROTCTL_STO_Msk | USCI_PROTCTL_AA_Msk);

    if (u8Start)
        u32Reg |= USCI_PROTCTL_STA_Msk;
    if (u8Stop)
        u32Reg |= USCI_PROTCTL_STO_Msk;
    if (u8Ptrg)
        u32Reg |= USCI_PROTCTL_PTRG_Msk;
    if (u8Ack)
        u32Reg |= USCI_PROTCTL_AA_Msk;

    usci->PROTCTL = u32Val | u32Reg;
}

/**
  * @brief This function disables the interrupt of USCI_I2C module.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @param[in] u32Mask The combination of all related interrupt enable bits.
  *                              Each bit corresponds to a interrupt enable bit.
  *                              This parameter decides which interrupts will be disabled. It is combination of:
  *                                      - \ref USCI_I2C_TO_INT_MASK
  *                                      - \ref USCI_I2C_STAR_INT_MASK
  *                                      - \ref USCI_I2C_STOR_INT_MASK
  *                                      - \ref USCI_I2C_NACK_INT_MASK
  *                                      - \ref USCI_I2C_ARBLO_INT_MASK
  *                                      - \ref USCI_I2C_ERR_INT_MASK
  *                                      - \ref USCI_I2C_ACK_INT_MASK
  * @return None
  */
void USCI_I2C_DisableInt(USCI_T *usci, uint32_t u32Mask)
{
    /* Disable time-out interrupt flag */
    if((u32Mask & USCI_I2C_TO_INT_MASK) == USCI_I2C_TO_INT_MASK)
        usci->PROTIEN &= ~USCI_PROTIEN_TOIEN_Msk;

    /* Disable start condition received interrupt flag */
    if((u32Mask & USCI_I2C_STAR_INT_MASK) == USCI_I2C_STAR_INT_MASK)
        usci->PROTIEN &= ~USCI_PROTIEN_STARIEN_Msk;

    /* Disable stop condition received interrupt flag */
    if((u32Mask & USCI_I2C_STOR_INT_MASK) == USCI_I2C_STOR_INT_MASK)
        usci->PROTIEN &= ~USCI_PROTIEN_STORIEN_Msk;

    /* Disable non-acknowledge interrupt flag */
    if((u32Mask & USCI_I2C_NACK_INT_MASK) == USCI_I2C_NACK_INT_MASK)
        usci->INTEN &= ~USCI_PROTIEN_NACKIEN_Msk;

    /* Disable arbitration lost interrupt flag */
    if((u32Mask & USCI_I2C_ARBLO_INT_MASK) == USCI_I2C_ARBLO_INT_MASK)
        usci->INTEN &= ~USCI_PROTIEN_ARBLOIEN_Msk;

    /* Disable error interrupt flag */
    if((u32Mask & USCI_I2C_ERR_INT_MASK) == USCI_I2C_ERR_INT_MASK)
        usci->INTEN &= ~USCI_PROTIEN_ERRIEN_Msk;

    /* Disable acknowledge interrupt flag */
    if((u32Mask & USCI_I2C_ACK_INT_MASK) == USCI_I2C_ACK_INT_MASK)
        usci->INTEN &= ~USCI_PROTIEN_ACKIEN_Msk;
}

/**
  * @brief This function enables the interrupt of USCI_I2C module.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @param[in] u32Mask The combination of all related interrupt enable bits.
  *                              Each bit corresponds to a interrupt enable bit.
  *                              This parameter decides which interrupts will be enabled. It is combination of:
  *                                      - \ref USCI_I2C_TO_INT_MASK
  *                                      - \ref USCI_I2C_STAR_INT_MASK
  *                                      - \ref USCI_I2C_STOR_INT_MASK
  *                                      - \ref USCI_I2C_NACK_INT_MASK
  *                                      - \ref USCI_I2C_ARBLO_INT_MASK
  *                                      - \ref USCI_I2C_ERR_INT_MASK
  *                                      - \ref USCI_I2C_ACK_INT_MASK
  * @return None
  */
void USCI_I2C_EnableInt(USCI_T *usci, uint32_t u32Mask)
{
    /* Enable time-out interrupt flag */
    if((u32Mask & USCI_I2C_TO_INT_MASK) == USCI_I2C_TO_INT_MASK)
        usci->PROTIEN |= USCI_PROTIEN_TOIEN_Msk;

    /* Enable start condition received interrupt flag */
    if((u32Mask & USCI_I2C_STAR_INT_MASK) == USCI_I2C_STAR_INT_MASK)
        usci->PROTIEN |= USCI_PROTIEN_STARIEN_Msk;

    /* Enable stop condition received interrupt flag */
    if((u32Mask & USCI_I2C_STOR_INT_MASK) == USCI_I2C_STOR_INT_MASK)
        usci->PROTIEN |= USCI_PROTIEN_STORIEN_Msk;

    /* Enable non-acknowledge interrupt flag */
    if((u32Mask & USCI_I2C_NACK_INT_MASK) == USCI_I2C_NACK_INT_MASK)
        usci->INTEN |= USCI_PROTIEN_NACKIEN_Msk;

    /* Enable arbitration lost interrupt flag */
    if((u32Mask & USCI_I2C_ARBLO_INT_MASK) == USCI_I2C_ARBLO_INT_MASK)
        usci->INTEN |= USCI_PROTIEN_ARBLOIEN_Msk;

    /* Enable error interrupt flag */
    if((u32Mask & USCI_I2C_ERR_INT_MASK) == USCI_I2C_ERR_INT_MASK)
        usci->INTEN |= USCI_PROTIEN_ERRIEN_Msk;

    /* Enable acknowledge interrupt flag */
    if((u32Mask & USCI_I2C_ACK_INT_MASK) == USCI_I2C_ACK_INT_MASK)
        usci->INTEN |= USCI_PROTIEN_ACKIEN_Msk;
}

/**
  * @brief This function returns the real bus clock of USCI_I2C module.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @return Actual USCI_I2C bus clock frequency.
  */
uint32_t USCI_I2C_GetBusClockFreq(USCI_T *usci)
{
    uint32_t u32Divider;

    u32Divider = (usci->BRGEN & USCI_BRGEN_CLKDIV_Msk) >> USCI_BRGEN_CLKDIV_Pos;

    return ( CLK_GetPCLKFreq() / ((u32Divider+1)<<1) );
}

/**
  * @brief This function sets bus frequency of USCI_I2C module.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @param[in] u32BusClock The target bus speed of USCI_I2C module.
  * @return Actual USCI_I2C bus clock frequency.
  */
uint32_t USCI_I2C_SetBusClockFreq(USCI_T *usci, uint32_t u32BusClock)
{
    uint32_t u32ClkDiv;
    uint32_t u32Pclk = CLK_GetPCLKFreq();

    u32ClkDiv = (uint32_t) ((((((u32Pclk/2)*10)/(u32BusClock))+5)/10)-1); /* Compute proper divider for USCI_I2C clock */

    /* Set USCI_I2C bus clock */
    usci->BRGEN &= ~USCI_BRGEN_CLKDIV_Msk;
    usci->BRGEN |=  (u32ClkDiv << USCI_BRGEN_CLKDIV_Pos);

    return ( u32Pclk / ((u32ClkDiv+1)<<1) );
}

/**
  * @brief This function gets the interrupt flag of USCI_I2C module.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @param[in] u32Mask The combination of all related interrupt sources.
  *                     Each bit corresponds to a interrupt source.
  *                     This parameter decides which interrupt flags will be read. It is combination of:
  *                                      - \ref USCI_I2C_TO_INT_MASK
  *                                      - \ref USCI_I2C_STAR_INT_MASK
  *                                      - \ref USCI_I2C_STOR_INT_MASK
  *                                      - \ref USCI_I2C_NACK_INT_MASK
  *                                      - \ref USCI_I2C_ARBLO_INT_MASK
  *                                      - \ref USCI_I2C_ERR_INT_MASK
  *                                      - \ref USCI_I2C_ACK_INT_MASK
  * @return       Interrupt flags of selected sources.
  */
uint32_t USCI_I2C_GetIntFlag(USCI_T *usci, uint32_t u32Mask)
{
    uint32_t u32IntFlag = 0;

    /* Check Time-out Interrupt Flag */
    if((u32Mask & USCI_I2C_TO_INT_MASK) && (usci->PROTSTS & USCI_PROTSTS_TOIF_Msk))
        u32IntFlag |= USCI_I2C_TO_INT_MASK;

    /* Check Start Condition Received Interrupt Flag */
    if((u32Mask & USCI_I2C_STAR_INT_MASK) && (usci->PROTSTS & USCI_PROTSTS_STARIF_Msk))
        u32IntFlag |= USCI_I2C_STAR_INT_MASK;

    /* Check Stop Condition Received Interrupt Flag */
    if((u32Mask & USCI_I2C_STOR_INT_MASK) && (usci->PROTSTS & USCI_PROTSTS_STORIF_Msk))
        u32IntFlag |= USCI_I2C_STOR_INT_MASK;

    /* Check Non-Acknowledge Interrupt Flag */
    if((u32Mask & USCI_I2C_NACK_INT_MASK) && (usci->PROTSTS & USCI_PROTSTS_NACKIF_Msk))
        u32IntFlag |= USCI_I2C_NACK_INT_MASK;

    /* Check Arbitration Lost Interrupt Flag */
    if((u32Mask & USCI_I2C_ARBLO_INT_MASK) && (usci->PROTSTS & USCI_PROTSTS_ARBLOIF_Msk))
        u32IntFlag |= USCI_I2C_ARBLO_INT_MASK;

    /* Check Error Interrupt Flag */
    if((u32Mask & USCI_I2C_ERR_INT_MASK) && (usci->PROTSTS & USCI_PROTSTS_ERRIF_Msk))
        u32IntFlag |= USCI_I2C_ERR_INT_MASK;

    /* Check Acknowledge Interrupt Flag */
    if((u32Mask & USCI_I2C_ACK_INT_MASK) && (usci->PROTSTS & USCI_PROTSTS_ACKIF_Msk))
        u32IntFlag |= USCI_I2C_ACK_INT_MASK;

    return u32IntFlag;
}

/**
  * @brief This function clears the interrupt flag of USCI_I2C module.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @param[in] u32Mask The combination of all related interrupt sources.
  *                     Each bit corresponds to a interrupt source.
  *                     This parameter decides which interrupt flags will be cleared. It is combination of:
  *                                      - \ref USCI_I2C_TO_INT_MASK
  *                                      - \ref USCI_I2C_STAR_INT_MASK
  *                                      - \ref USCI_I2C_STOR_INT_MASK
  *                                      - \ref USCI_I2C_NACK_INT_MASK
  *                                      - \ref USCI_I2C_ARBLO_INT_MASK
  *                                      - \ref USCI_I2C_ERR_INT_MASK
  *                                      - \ref USCI_I2C_ACK_INT_MASK
  * @return None
  */
void USCI_I2C_ClearIntFlag(USCI_T* usci , uint32_t u32Mask)
{
    /* Cleare Time-out Interrupt Flag */
    if(u32Mask & USCI_I2C_TO_INT_MASK)
        usci->PROTSTS = USCI_PROTSTS_TOIF_Msk;

    /* Cleare Start Condition Received Interrupt Flag */
    if(u32Mask & USCI_I2C_STAR_INT_MASK)
        usci->PROTSTS = USCI_PROTSTS_STARIF_Msk;

    /* Cleare Stop Condition Received Interrupt Flag */
    if(u32Mask & USCI_I2C_STOR_INT_MASK)
        usci->PROTSTS = USCI_PROTSTS_STORIF_Msk;

    /* Cleare Non-Acknowledge Interrupt Flag */
    if(u32Mask & USCI_I2C_NACK_INT_MASK)
        usci->BUFSTS = USCI_PROTSTS_NACKIF_Msk;

    /* Cleare Arbitration Lost Interrupt Flag */
    if(u32Mask & USCI_I2C_ARBLO_INT_MASK)
        usci->PROTSTS = USCI_PROTSTS_ARBLOIF_Msk;

    /* Cleare Error Interrupt Flag */
    if(u32Mask & USCI_I2C_ERR_INT_MASK)
        usci->PROTSTS = USCI_PROTSTS_ERRIF_Msk;

    /* Cleare Acknowledge Interrupt Flag */
    if(u32Mask & USCI_I2C_ACK_INT_MASK)
        usci->PROTSTS = USCI_PROTSTS_ACKIF_Msk;
}

/**
  * @brief This function returns the data stored in data register of USCI_I2C module.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @return USCI_I2C data.
  */
uint32_t USCI_I2C_GetData(USCI_T *usci)
{
    return ( usci->RXDAT );
}

/**
  * @brief This function writes the data to data register of USCI_I2C module.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @param[in] u8Data The data which will be written to data register of USCI_I2C module.
  * @return None
  */
void USCI_I2C_SetData(USCI_T *usci, uint8_t u8Data)
{
    usci->TXDAT = u8Data;
}

/**
  * @brief Configure slave address and enable GC mode.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @param[in] u8SlaveNo This parameter is not used.
  * @param[in] u16SlaveAddr The slave address.
  * @param[in] u8GCMode GC mode enable or not. Valid values are:
  *              - \ref USCI_I2C_GCMODE_ENABLE
  *              - \ref USCI_I2C_GCMODE_DISABLE
  * @return None
  */
void USCI_I2C_SetSlaveAddr(USCI_T *usci, uint8_t u8SlaveNo, uint16_t u16SlaveAddr, uint8_t u8GCMode)
{
    usci->DEVADDR0  = u16SlaveAddr;
    usci->PROTCTL  = (usci->PROTCTL & ~USCI_PROTCTL_GCFUNC_Msk) |u8GCMode;
}

/**
  * @brief Configure the mask of slave address. The corresponding address bit is "Don't Care".
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @param[in] u8SlaveNo This parameter is not used.
  * @param[in] u16SlaveAddrMask The slave address mask.
  * @return None
  */
void USCI_I2C_SetSlaveAddrMask(USCI_T *usci, uint8_t u8SlaveNo, uint16_t u16SlaveAddrMask)
{
    usci->ADDRMSK0  = u16SlaveAddrMask;
}

/**
  * @brief This function enables timeout function and configures timeout counter.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @param[in] u32TimeoutCnt Timeout counter. Valid values are between 0~0x3FF
  * @return None
  */
void USCI_I2C_EnableTimeout(USCI_T *usci, uint32_t u32TimeoutCnt)
{
    usci->PROTCTL = (usci->PROTCTL & ~USCI_PROTCTL_TOCNT_Msk) | (u32TimeoutCnt << USCI_PROTCTL_TOCNT_Pos);
    usci->BRGEN = (usci->BRGEN & ~USCI_BRGEN_TMCNTSRC_Msk) | USCI_BRGEN_TMCNTEN_Msk;
}

/**
  * @brief This function disables time-out function.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @return None
  */
void USCI_I2C_DisableTimeout(USCI_T *usci)
{
    usci->PROTCTL &= ~USCI_PROTCTL_TOCNT_Msk;
    usci->BRGEN &= ~USCI_BRGEN_TMCNTEN_Msk;
}

/**
  * @brief This function enables the wakeup function of USCI_I2C module.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @param[in] u8WakeupMode The wake-up mode selection. Valid values are:
  *              - \ref USCI_I2C_DATA_TOGGLE_WK
  *              - \ref USCI_I2C_ADDR_MATCH_WK
  * @return None
  */
void USCI_I2C_EnableWakeup(USCI_T *usci, uint8_t u8WakeupMode)
{
    usci->WKCTL = (usci->WKCTL & ~USCI_WKCTL_WKADDREN_Msk) | (u8WakeupMode | USCI_WKCTL_WKEN_Msk);
}

/**
  * @brief This function disables the wakeup function of USCI_I2C module.
  * @param[in] usci The pointer of the specified USCI_I2C module.
  * @return None
  */
void USCI_I2C_DisableWakeup(USCI_T *usci)
{
    usci->WKCTL &= ~USCI_WKCTL_WKEN_Msk;
}

/*@}*/ /* end of group NM1120_USCI_I2C_EXPORTED_FUNCTIONS */

/*@}*/ /* end of group NM1120_USCI_I2C_Driver */

/*@}*/ /* end of group NM1120_Device_Driver */

/*** (C) COPYRIGHT 2016 Nuvoton Technology Corp. ***/
