/****************************************************************************//**
 * @file     usci_spi.c
 * @version  V0.10
 * $Revision: 4 $
 * $Date: 16/03/30 10:06a $
 * @brief    NM1120 USCI_SPI driver source file
 *
 * @note
 * Copyright (C) 2016 Nuvoton Technology Corp. All rights reserved.
*****************************************************************************/
#include "NM1120.h"

/** @addtogroup NM1120_Device_Driver NM1120 Device Driver
  @{
*/

/** @addtogroup NM1120_USCI_SPI_Driver USCI_SPI Driver
  @{
*/


/** @addtogroup NM1120_USCI_SPI_EXPORTED_FUNCTIONS USCI_SPI Exported Functions
  @{
*/

/**
  * @brief  This function make USCI_SPI module be ready to transfer.
  *         By default, the USCI_SPI transfer sequence is MSB first and
  *         the automatic slave select function is disabled. In
  *         Slave mode, the u32BusClock must be NULL and the USCI_SPI clock
  *         divider setting will be 0.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @param[in]  u32MasterSlave Decide the USCI_SPI module is operating in master mode or in slave mode. Valid values are:
  *                    - \ref USCI_SPI_SLAVE
  *                    - \ref USCI_SPI_MASTER
  * @param[in]  u32SPIMode Decide the transfer timing. Valid values are:
  *                    - \ref USCI_SPI_MODE_0
  *                    - \ref USCI_SPI_MODE_1
  *                    - \ref USCI_SPI_MODE_2
  *                    - \ref USCI_SPI_MODE_3
  * @param[in]  u32DataWidth The data width of a USCI_SPI transaction.
  * @param[in]  u32BusClock The expected frequency of USCI_SPI bus clock in Hz.
  * @return Actual frequency of USCI_SPI peripheral clock.
  */
uint32_t USCI_SPI_Open(USCI_T *usci, uint32_t u32MasterSlave, uint32_t u32SPIMode,  uint32_t u32DataWidth, uint32_t u32BusClock)
{
    uint32_t u32ClkDiv = 0;
    uint32_t u32Pclk = CLK_GetPCLKFreq();

    if(u32BusClock != 0)
        u32ClkDiv = (uint32_t) ((((((u32Pclk/2)*10)/(u32BusClock))+5)/10)-1); /* Compute proper divider for USCI_SPI clock */

    /* Enable USCI_SPI protocol */
    usci->CTL &= ~USCI_CTL_FUNMODE_Msk;
    usci->CTL = 1 << USCI_CTL_FUNMODE_Pos;

    /* Data format configuration */
    if(u32DataWidth == 16)
        u32DataWidth = 0;
    usci->LINECTL &= ~USCI_LINECTL_DWIDTH_Msk;
    usci->LINECTL |= (u32DataWidth << USCI_LINECTL_DWIDTH_Pos);

    /* MSB data format */
    usci->LINECTL &= ~USCI_LINECTL_LSB_Msk;

    /* Set operating mode and transfer timing */
    usci->PROTCTL &= ~(USCI_PROTCTL_SCLKMODE_Msk | USCI_PROTCTL_AUTOSS_Msk | USCI_PROTCTL_SLAVE_Msk);
    usci->PROTCTL |= (u32MasterSlave | u32SPIMode);

    /* Set USCI_SPI bus clock */
    usci->BRGEN &= ~USCI_BRGEN_CLKDIV_Msk;
    usci->BRGEN |=  (u32ClkDiv << USCI_BRGEN_CLKDIV_Pos);
    usci->PROTCTL |=  USCI_PROTCTL_PROTEN_Msk;

    if(u32BusClock != 0)
        return ( u32Pclk / ((u32ClkDiv+1)<<1) );
    else
        return 0;
}

/**
  * @brief Disable USCI_SPI function mode.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @return None
  */
void USCI_SPI_Close(USCI_T *usci)
{
    usci->CTL &= ~USCI_CTL_FUNMODE_Msk;
}

/**
  * @brief Clear Rx buffer.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @return None
  */
void USCI_SPI_ClearRxBuf(USCI_T *usci)
{
    usci->BUFCTL |= USCI_BUFCTL_RXCLR_Msk;
}

/**
  * @brief Clear Tx buffer.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @return None
  */
void USCI_SPI_ClearTxBuf(USCI_T *usci)
{
    usci->BUFCTL |= USCI_BUFCTL_TXCLR_Msk;
}

/**
  * @brief Disable the automatic slave select function.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @return None
  */
void USCI_SPI_DisableAutoSS(USCI_T *usci)
{
    usci->PROTCTL &= ~(USCI_PROTCTL_AUTOSS_Msk | USCI_PROTCTL_SS_Msk);
}

/**
  * @brief Enable the automatic slave select function. Only available in Master mode.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @param[in]  u32SSPinMask This parameter is not used.
  * @param[in]  u32ActiveLevel The active level of slave select signal. Valid values are:
  *                     - \ref USCI_SPI_SS_ACTIVE_HIGH
  *                     - \ref USCI_SPI_SS_ACTIVE_LOW
  * @return None
  */
void USCI_SPI_EnableAutoSS(USCI_T *usci, uint32_t u32SSPinMask, uint32_t u32ActiveLevel)
{
    usci->CTLIN0 = (usci->CTLIN0 & ~USCI_CTLIN0_ININV_Msk) | u32ActiveLevel;
    usci->PROTCTL |= USCI_PROTCTL_AUTOSS_Msk;
}

/**
  * @brief Set the USCI_SPI bus clock. Only available in Master mode.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @param[in]  u32BusClock The expected frequency of USCI_SPI bus clock.
  * @return Actual frequency of USCI_SPI peripheral clock.
  */
uint32_t USCI_SPI_SetBusClock(USCI_T *usci, uint32_t u32BusClock)
{
    uint32_t u32ClkDiv;
    uint32_t u32Pclk = CLK_GetPCLKFreq();

    u32ClkDiv = (uint32_t) ((((((u32Pclk/2)*10)/(u32BusClock))+5)/10)-1); /* Compute proper divider for USCI_SPI clock */

    /* Set USCI_SPI bus clock */
    usci->BRGEN &= ~USCI_BRGEN_CLKDIV_Msk;
    usci->BRGEN |=  (u32ClkDiv << USCI_BRGEN_CLKDIV_Pos);

    return ( u32Pclk / ((u32ClkDiv+1)<<1) );
}

/**
  * @brief Get the actual frequency of USCI_SPI bus clock. Only available in Master mode.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @return Actual USCI_SPI bus clock frequency.
  */
uint32_t USCI_SPI_GetBusClock(USCI_T *usci)
{
    uint32_t u32ClkDiv;

    u32ClkDiv = (usci->BRGEN & USCI_BRGEN_CLKDIV_Msk) >> USCI_BRGEN_CLKDIV_Pos;

    return ( CLK_GetPCLKFreq() / ((u32ClkDiv+1)<<1) );
}

/**
  * @brief Enable related interrupts specified by u32Mask parameter.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @param[in]  u32Mask The combination of all related interrupt enable bits.
  *         Each bit corresponds to a interrupt bit.
  *         This parameter decides which interrupts will be enabled. Valid values are:
  *           - \ref USCI_SPI_SSINAIEN_MASK
  *           - \ref USCI_SPI_SSACTIEN_MASK
  *           - \ref USCI_SPI_SLVTOIEN_MASK
  *           - \ref USCI_SPI_SLVBEIEN_MASK
  *           - \ref USCI_SPI_TXUDRIEN_MASK
  *           - \ref USCI_SPI_RXOVIEN_MASK
  *           - \ref USCI_SPI_TXSTIEN_MASK
  *           - \ref USCI_SPI_TXENDIEN_MASK
  *           - \ref USCI_SPI_RXSTIEN_MASK
  *           - \ref USCI_SPI_RXENDIEN_MASK
  * @return None
  */
void USCI_SPI_EnableInt(USCI_T *usci, uint32_t u32Mask)
{
    /* Enable slave selection signal inactive interrupt flag */
    if((u32Mask & USCI_SPI_SSINAIEN_MASK) == USCI_SPI_SSINAIEN_MASK)
        usci->PROTIEN |= USCI_PROTIEN_SSINAIEN_Msk;

    /* Enable slave selection signal active interrupt flag */
    if((u32Mask & USCI_SPI_SSACTIEN_MASK) == USCI_SPI_SSACTIEN_MASK)
        usci->PROTIEN |= USCI_PROTIEN_SSACTIEN_Msk;

    /* Enable slave time-out interrupt flag */
    if((u32Mask & USCI_SPI_SLVTOIEN_MASK) == USCI_SPI_SLVTOIEN_MASK)
        usci->PROTIEN |= USCI_PROTIEN_SLVTOIEN_Msk;

    /* Enable slave bit count error interrupt flag */
    if((u32Mask & USCI_SPI_SLVBEIEN_MASK) == USCI_SPI_SLVBEIEN_MASK)
        usci->PROTIEN |= USCI_PROTIEN_SLVBEIEN_Msk;

    /* Enable TX under run interrupt flag */
    if((u32Mask & USCI_SPI_TXUDRIEN_MASK) == USCI_SPI_TXUDRIEN_MASK)
        usci->BUFCTL |= USCI_BUFCTL_TXUDRIEN_Msk;

    /* Enable RX overrun interrupt flag */
    if((u32Mask & USCI_SPI_RXOVIEN_MASK) == USCI_SPI_RXOVIEN_MASK)
        usci->BUFCTL |= USCI_BUFCTL_RXOVIEN_Msk;

    /* Enable TX start interrupt flag */
    if((u32Mask & USCI_SPI_TXSTIEN_MASK) == USCI_SPI_TXSTIEN_MASK)
        usci->INTEN |= USCI_INTEN_TXSTIEN_Msk;

    /* Enable TX end interrupt flag */
    if((u32Mask & USCI_SPI_TXENDIEN_MASK) == USCI_SPI_TXENDIEN_MASK)
        usci->INTEN |= USCI_INTEN_TXENDIEN_Msk;

    /* Enable RX start interrupt flag */
    if((u32Mask & USCI_SPI_RXSTIEN_MASK) == USCI_SPI_RXSTIEN_MASK)
        usci->INTEN |= USCI_INTEN_RXSTIEN_Msk;

    /* Enable RX end interrupt flag */
    if((u32Mask & USCI_SPI_RXENDIEN_MASK) == USCI_SPI_RXENDIEN_MASK)
        usci->INTEN |= USCI_INTEN_RXENDIEN_Msk;
}

/**
  * @brief Disable related interrupts specified by u32Mask parameter.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @param[in]  u32Mask The combination of all related interrupt enable bits.
  *         Each bit corresponds to a interrupt bit.
  *         This parameter decides which interrupts will be disabled. Valid values are:
  *           - \ref USCI_SPI_SSINAIEN_MASK
  *           - \ref USCI_SPI_SSACTIEN_MASK
  *           - \ref USCI_SPI_SLVTOIEN_MASK
  *           - \ref USCI_SPI_SLVBEIEN_MASK
  *           - \ref USCI_SPI_TXUDRIEN_MASK
  *           - \ref USCI_SPI_RXOVIEN_MASK
  *           - \ref USCI_SPI_TXSTIEN_MASK
  *           - \ref USCI_SPI_TXENDIEN_MASK
  *           - \ref USCI_SPI_RXSTIEN_MASK
  *           - \ref USCI_SPI_RXENDIEN_MASK
  * @return None
  */
void USCI_SPI_DisableInt(USCI_T *usci, uint32_t u32Mask)
{
    /* Disable slave selection signal inactive interrupt flag */
    if((u32Mask & USCI_SPI_SSINAIEN_MASK) == USCI_SPI_SSINAIEN_MASK)
        usci->PROTIEN &= ~USCI_PROTIEN_SSINAIEN_Msk;

    /* Disable slave selection signal active interrupt flag */
    if((u32Mask & USCI_SPI_SSACTIEN_MASK) == USCI_SPI_SSACTIEN_MASK)
        usci->PROTIEN &= ~USCI_PROTIEN_SSACTIEN_Msk;

    /* Disable slave time-out interrupt flag */
    if((u32Mask & USCI_SPI_SLVTOIEN_MASK) == USCI_SPI_SLVTOIEN_MASK)
        usci->PROTIEN &= ~USCI_PROTIEN_SLVTOIEN_Msk;

    /* Disable slave bit count error interrupt flag */
    if((u32Mask & USCI_SPI_SLVBEIEN_MASK) == USCI_SPI_SLVBEIEN_MASK)
        usci->PROTIEN &= ~USCI_PROTIEN_SLVBEIEN_Msk;

    /* Disable TX under run interrupt flag */
    if((u32Mask & USCI_SPI_TXUDRIEN_MASK) == USCI_SPI_TXUDRIEN_MASK)
        usci->BUFCTL &= ~USCI_BUFCTL_TXUDRIEN_Msk;

    /* Disable RX overrun interrupt flag */
    if((u32Mask & USCI_SPI_RXOVIEN_MASK) == USCI_SPI_RXOVIEN_MASK)
        usci->BUFCTL &= ~USCI_BUFCTL_RXOVIEN_Msk;

    /* Disable TX start interrupt flag */
    if((u32Mask & USCI_SPI_TXSTIEN_MASK) == USCI_SPI_TXSTIEN_MASK)
        usci->INTEN &= ~USCI_INTEN_TXSTIEN_Msk;

    /* Disable TX end interrupt flag */
    if((u32Mask & USCI_SPI_TXENDIEN_MASK) == USCI_SPI_TXENDIEN_MASK)
        usci->INTEN &= ~USCI_INTEN_TXENDIEN_Msk;

    /* Disable RX start interrupt flag */
    if((u32Mask & USCI_SPI_RXSTIEN_MASK) == USCI_SPI_RXSTIEN_MASK)
        usci->INTEN &= ~USCI_INTEN_RXSTIEN_Msk;

    /* Disable RX end interrupt flag */
    if((u32Mask & USCI_SPI_RXENDIEN_MASK) == USCI_SPI_RXENDIEN_MASK)
        usci->INTEN &= ~USCI_INTEN_RXENDIEN_Msk;
}

/**
  * @brief  Get interrupt flag.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @param[in]  u32Mask The combination of all related interrupt sources.
  *         Each bit corresponds to a interrupt source.
  *         This parameter decides which interrupt flags will be read. It is combination of:
  *           - \ref USCI_SPI_SSINAIEN_MASK
  *           - \ref USCI_SPI_SSACTIEN_MASK
  *           - \ref USCI_SPI_SLVTOIEN_MASK
  *           - \ref USCI_SPI_SLVBEIEN_MASK
  *           - \ref USCI_SPI_TXUDRIEN_MASK
  *           - \ref USCI_SPI_RXOVIEN_MASK
  *           - \ref USCI_SPI_TXSTIEN_MASK
  *           - \ref USCI_SPI_TXENDIEN_MASK
  *           - \ref USCI_SPI_RXSTIEN_MASK
  *           - \ref USCI_SPI_RXENDIEN_MASK
  * @return Interrupt flags of selected sources.
  */
uint32_t USCI_SPI_GetIntFlag(USCI_T *usci, uint32_t u32Mask)
{
    uint32_t u32IntFlag = 0;

    /* Check slave selection signal inactive interrupt flag */
    if((u32Mask & USCI_SPI_SSINAIEN_MASK) && (usci->PROTSTS & USCI_PROTSTS_SSINAIF_Msk))
        u32IntFlag |= USCI_SPI_SSINAIEN_MASK;

    /* Check slave selection signal active interrupt flag */
    if((u32Mask & USCI_SPI_SSACTIEN_MASK) && (usci->PROTSTS & USCI_PROTSTS_SSACTIF_Msk))
        u32IntFlag |= USCI_SPI_SSACTIEN_MASK;

    /* Check slave time-out interrupt flag */
    if((u32Mask & USCI_SPI_SLVTOIEN_MASK) && (usci->PROTSTS & USCI_PROTSTS_SLVTOIF_Msk))
        u32IntFlag |= USCI_SPI_SLVTOIEN_MASK;

    /* Check slave bit count error interrupt flag */
    if((u32Mask & USCI_SPI_SLVBEIEN_MASK) && (usci->PROTSTS & USCI_PROTSTS_SLVBEIF_Msk))
        u32IntFlag |= USCI_SPI_SLVBEIEN_MASK;

    /* Check TX under run interrupt flag */
    if((u32Mask & USCI_SPI_TXUDRIEN_MASK) && (usci->BUFSTS & USCI_BUFSTS_TXUDRIF_Msk))
        u32IntFlag |= USCI_SPI_TXUDRIEN_MASK;

    /* Check RX overrun interrupt flag */
    if((u32Mask & USCI_SPI_RXOVIEN_MASK) && (usci->BUFSTS & USCI_BUFSTS_RXOVIF_Msk))
        u32IntFlag |= USCI_SPI_RXOVIEN_MASK;

    /* Check TX start interrupt flag */
    if((u32Mask & USCI_SPI_TXSTIEN_MASK) && (usci->PROTSTS & USCI_PROTSTS_TXSTIF_Msk))
        u32IntFlag |= USCI_SPI_TXSTIEN_MASK;

    /* Check TX end interrupt flag */
    if((u32Mask & USCI_SPI_TXENDIEN_MASK) && (usci->PROTSTS & USCI_PROTSTS_TXENDIF_Msk))
        u32IntFlag |= USCI_SPI_TXENDIEN_MASK;

    /* Check RX start interrupt flag */
    if((u32Mask & USCI_SPI_RXSTIEN_MASK) && (usci->PROTSTS & USCI_PROTSTS_RXSTIF_Msk))
        u32IntFlag |= USCI_SPI_RXSTIEN_MASK;

    /* Check RX end interrupt flag */
    if((u32Mask & USCI_SPI_RXENDIEN_MASK) && (usci->PROTSTS & USCI_PROTSTS_RXENDIF_Msk))
        u32IntFlag |= USCI_SPI_RXENDIEN_MASK;

    return u32IntFlag;
}

/**
  * @brief  Clear interrupt flag.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @param[in]  u32Mask The combination of all related interrupt sources.
  *         Each bit corresponds to a interrupt source.
  *         This parameter decides which interrupt flags will be cleared. It could be the combination of:
  *           - \ref USCI_SPI_SSINAIEN_MASK
  *           - \ref USCI_SPI_SSACTIEN_MASK
  *           - \ref USCI_SPI_SLVTOIEN_MASK
  *           - \ref USCI_SPI_SLVBEIEN_MASK
  *           - \ref USCI_SPI_TXUDRIEN_MASK
  *           - \ref USCI_SPI_RXOVIEN_MASK
  *           - \ref USCI_SPI_TXSTIEN_MASK
  *           - \ref USCI_SPI_TXENDIEN_MASK
  *           - \ref USCI_SPI_RXSTIEN_MASK
  *           - \ref USCI_SPI_RXENDIEN_MASK
  * @return None
  */
void USCI_SPI_ClearIntFlag(USCI_T *usci, uint32_t u32Mask)
{
    /* Clear slave selection signal inactive interrupt flag */
    if(u32Mask & USCI_SPI_SSINAIEN_MASK)
        usci->PROTSTS = USCI_PROTSTS_SSINAIF_Msk;

    /* Clear slave selection signal active interrupt flag */
    if(u32Mask & USCI_SPI_SSACTIEN_MASK)
        usci->PROTSTS = USCI_PROTSTS_SSACTIF_Msk;

    /* Clear slave time-out interrupt flag */
    if(u32Mask & USCI_SPI_SLVTOIEN_MASK)
        usci->PROTSTS = USCI_PROTSTS_SLVTOIF_Msk;

    /* Clear slave bit count error interrupt flag */
    if(u32Mask & USCI_SPI_SLVBEIEN_MASK)
        usci->PROTSTS = USCI_PROTSTS_SLVBEIF_Msk;

    /* Clear TX under run interrupt flag */
    if(u32Mask & USCI_SPI_TXUDRIEN_MASK)
        usci->BUFSTS = USCI_BUFSTS_TXUDRIF_Msk;

    /* Clear RX overrun interrupt flag */
    if(u32Mask & USCI_SPI_RXOVIEN_MASK)
        usci->BUFSTS = USCI_BUFSTS_RXOVIF_Msk;

    /* Clear TX start interrupt flag */
    if(u32Mask & USCI_SPI_TXSTIEN_MASK)
        usci->PROTSTS = USCI_PROTSTS_TXSTIF_Msk;

    /* Clear TX end interrupt flag */
    if(u32Mask & USCI_SPI_TXENDIEN_MASK)
        usci->PROTSTS = USCI_PROTSTS_TXENDIF_Msk;

    /* Clear RX start interrupt flag */
    if(u32Mask & USCI_SPI_RXSTIEN_MASK)
        usci->PROTSTS = USCI_PROTSTS_RXSTIF_Msk;

    /* Clear RX end interrupt flag */
    if(u32Mask & USCI_SPI_RXENDIEN_MASK)
        usci->PROTSTS = USCI_PROTSTS_RXENDIF_Msk;
}

/**
  * @brief  Get USCI_SPI status.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @param[in]  u32Mask The combination of all related sources.
  *         Each bit corresponds to a source.
  *         This parameter decides which flags will be read. It is combination of:
  *           - \ref SPI_BUSY_MASK
  *           - \ref SPI_RX_EMPTY_MASK
  *           - \ref SPI_RX_FULL_MASK
  *           - \ref SPI_TX_EMPTY_MASK
  *           - \ref SPI_TX_FULL_MASK
  *           - \ref SPI_SSLINE_STS_MASK
  * @return Flags of selected sources.
  */
uint32_t USCI_SPI_GetStatus(USCI_T *usci, uint32_t u32Mask)
{
    uint32_t u32Flag = 0;

    /* Check busy status */
    if((u32Mask & SPI_BUSY_MASK) && (usci->PROTSTS & USCI_PROTSTS_BUSY_Msk))
        u32Flag |= SPI_BUSY_MASK;

    /* Check RX empty flag */
    if((u32Mask & SPI_RX_EMPTY_MASK) && (usci->BUFSTS & USCI_BUFSTS_RXEMPTY_Msk))
        u32Flag |= SPI_RX_EMPTY_MASK;

    /* Check RX full flag */
    if((u32Mask & SPI_RX_FULL_MASK) && (usci->BUFSTS & USCI_BUFSTS_RXFULL_Msk))
        u32Flag |= SPI_RX_FULL_MASK;

    /* Check TX empty flag */
    if((u32Mask & SPI_TX_EMPTY_MASK) && (usci->BUFSTS & USCI_BUFSTS_TXEMPTY_Msk))
        u32Flag |= SPI_TX_EMPTY_MASK;

    /* Check TX full flag */
    if((u32Mask & SPI_TX_FULL_MASK) && (usci->BUFSTS & USCI_BUFSTS_TXFULL_Msk))
        u32Flag |= SPI_TX_FULL_MASK;

    /* Check USCI_SPI_SS line status */
    if((u32Mask & SPI_SSLINE_STS_MASK) && (usci->PROTSTS & USCI_PROTSTS_SSLINE_Msk))
        u32Flag |= SPI_SSLINE_STS_MASK;

    return u32Flag;
}

/**
  * @brief  Enable USCI_SPI Wake-up Function.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @return None
  */
void USCI_SPI_EnableWakeup(USCI_T *usci)
{
    usci->WKCTL |= USCI_WKCTL_WKEN_Msk;
}

/**
  * @brief  Disable USCI_SPI Wake-up Function.
  * @param[in]  usci The pointer of the specified USCI_SPI module.
  * @return None
  */
void USCI_SPI_DisableWakeup(USCI_T *usci)
{
    usci->WKCTL &= ~USCI_WKCTL_WKEN_Msk;
}

/*@}*/ /* end of group NM1120_USCI_SPI_EXPORTED_FUNCTIONS */

/*@}*/ /* end of group NM1120_USCI_SPI_Driver */

/*@}*/ /* end of group NM1120_Device_Driver */

/*** (C) COPYRIGHT 2016 Nuvoton Technology Corp. ***/
