/******************************************************************************
 * @file     main.c
 * @version  V1.00
 * $Revision: 2 $
 * $Date: 16/03/31 7:18p $
 * @brief    Show how to set USCI_I2C in Slave mode and receive the data from Master.
 *           This sample code needs to work with USCI_I2C_Master.
 * @note
 * Copyright (C) 2016 Nuvoton Technology Corp. All rights reserved.
*****************************************************************************/
#include <stdio.h>
#include "NM1120.h"


/*---------------------------------------------------------------------------------------------------------*/
/* Global variables                                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
volatile uint32_t slave_buff_addr;
uint8_t g_au8SlvData[256];
uint8_t g_au8RxData[3];
volatile uint8_t g_u8DeviceAddr;
volatile uint8_t g_u8DataLenS;
enum USCI_I2C_SLAVE_EVENT s_Event;

typedef void (*USCI_I2C_FUNC)(uint32_t u32Status);

static USCI_I2C_FUNC s_USCI_I2CSHandlerFn = NULL;

/*---------------------------------------------------------------------------------------------------------*/
/*  USCI_I2C1 IRQ Handler                                                                                       */
/*---------------------------------------------------------------------------------------------------------*/
void USCI1_IRQHandler(void)
{
    uint32_t u32Status;

    u32Status = USCI_I2C_GET_PROT_STATUS(USCI1);

    if (USCI_I2C_GET_TIMEOUT_FLAG(USCI1)) {
        /* Clear USCI_I2C1 Timeout Flag */
        USCI_I2C_ClearTimeoutFlag(USCI1);
    } else {
        if (s_USCI_I2CSHandlerFn != NULL)
            s_USCI_I2CSHandlerFn(u32Status);
    }
}

/*---------------------------------------------------------------------------------------------------------*/
/*  USCI_I2C TRx Callback Function                                                                               */
/*---------------------------------------------------------------------------------------------------------*/
void USCI_I2C_SlaveTRx(uint32_t u32Status)
{
    uint8_t u8RxData;

    if((u32Status & USCI_PROTSTS_STARIF_Msk) == USCI_PROTSTS_STARIF_Msk) {
        USCI_I2C_CLR_PROT_INT_FLAG(USCI1, USCI_PROTSTS_STARIF_Msk); /* Clear START INT Flag */
        g_u8DataLenS = 0;
        s_Event = SLAVE_ADDRESS_ACK;
        USCI_I2C_SET_CONTROL_REG(USCI1, (USCI_I2C_CTL_PTRG | USCI_I2C_CTL_AA));
    } else if((u32Status & USCI_PROTSTS_ACKIF_Msk) == USCI_PROTSTS_ACKIF_Msk) {
        USCI_I2C_CLR_PROT_INT_FLAG(USCI1, USCI_PROTSTS_ACKIF_Msk);  /* Clear ACK INT Flag */
        if(s_Event == SLAVE_ADDRESS_ACK) {
            g_u8DataLenS = 0;
            if((USCI1->PROTSTS & USCI_PROTSTS_SLAREAD_Msk) == USCI_PROTSTS_SLAREAD_Msk) {   /* Own SLA+R has been receive; ACK has been return */
                s_Event = SLAVE_SEND_DATA;
                USCI1->TXDAT = g_au8SlvData[slave_buff_addr];
                slave_buff_addr++;
            } else {        /* Own SLA+W has been receive; ACK has been return */
                s_Event = SLAVE_GET_DATA;
            }
            g_u8DeviceAddr = (unsigned char)(USCI1->RXDAT);
        } else if(s_Event == SLAVE_GET_DATA) {
            u8RxData = (unsigned char)(USCI1->RXDAT);
            g_au8RxData[g_u8DataLenS] = u8RxData;
            g_u8DataLenS++;

            if(g_u8DataLenS == 2) { /* Address has been received; ACK has been returned*/
                slave_buff_addr = (g_au8RxData[0] << 8) + g_au8RxData[1];
            }
            if(g_u8DataLenS == 3) { /* Data has been received; ACK has been returned*/
                g_au8SlvData[slave_buff_addr] = g_au8RxData[2];
                g_u8DataLenS = 0;
            }
        }
        USCI_I2C_SET_CONTROL_REG(USCI1, (USCI_I2C_CTL_PTRG | USCI_I2C_CTL_AA));
    } else if((u32Status & USCI_PROTSTS_NACKIF_Msk) == USCI_PROTSTS_NACKIF_Msk) {   /* Data in TXDAT has been transmitted; NACK has been received */
        USCI_I2C_CLR_PROT_INT_FLAG(USCI1, USCI_PROTSTS_NACKIF_Msk); /* Clear NACK INT Flag */
        g_u8DataLenS = 0;
        s_Event = SLAVE_ADDRESS_ACK;
        USCI_I2C_SET_CONTROL_REG(USCI1, (USCI_I2C_CTL_PTRG | USCI_I2C_CTL_AA));
    } else if((u32Status & USCI_PROTSTS_STORIF_Msk) == USCI_PROTSTS_STORIF_Msk) {
        USCI_I2C_CLR_PROT_INT_FLAG(USCI1, USCI_PROTSTS_STORIF_Msk); /* Clear STOP INT Flag */
        g_u8DataLenS = 0;
        s_Event = SLAVE_ADDRESS_ACK;
        USCI_I2C_SET_CONTROL_REG(USCI1, (USCI_I2C_CTL_PTRG | USCI_I2C_CTL_AA));
    }
}

void SYS_Init(void)
{
    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Enable 48MHz HIRC */
    CLK_EnableXtalRC(CLK_PWRCTL_HIRCEN_Msk);

    /* Waiting for 48MHz clock ready */
    CLK_WaitClockReady(CLK_STATUS_HIRCSTB_Msk);

    /* HCLK Clock source from HIRC */
    CLK_SetHCLK(CLK_HCLK_SRC_HIRC,CLK_CLKDIV_HCLK(1));

    /* Enable IP clock */
    CLK_EnableModuleClock(USCI0_MODULE);
    CLK_EnableModuleClock(USCI1_MODULE);

    /* Update System Core Clock */
    /* User can use SystemCoreClockUpdate() to calculate SystemCoreClock and cyclesPerUs automatically. */
    SystemCoreClockUpdate();

    /* USCI-Uart0-GPD5(TX) + GPD6(RX) */
    /* Set GPD multi-function pins for USCI UART0 GPD5(TX) and GPD6(RX) */
    SYS->GPD_MFP = SYS->GPD_MFP & ~(SYS_GPD_MFP_PD5MFP_Msk | SYS_GPD_MFP_PD6MFP_Msk) | (SYS_GPD_MFP_PD5_UART0_TXD | SYS_GPD_MFP_PD6_UART0_RXD);

    /* Set GPD5 as output mode and GPD6 as Input mode */
    GPIO_SetMode(PD, BIT5, GPIO_MODE_OUTPUT);
    GPIO_SetMode(PD, BIT6, GPIO_MODE_INPUT);

    /* Set GPC multi-function pins for USCI I2C1 GPC0(SCL) and GPC2(SDA) */
    SYS->GPC_MFP = SYS->GPC_MFP & ~(SYS_GPC_MFP_PC0MFP_Msk | SYS_GPC_MFP_PC2MFP_Msk) | (SYS_GPC_MFP_PC0_I2C1_SCL | SYS_GPC_MFP_PC2_I2C1_SDA);

    /* Lock protected registers */
    SYS_LockReg();
}

void USCI_I2C1_Init(void)
{
    /* Open USCI_I2C1 and set clock to 100k */
    USCI_I2C_Open(USCI1, 100000);

    /* Get USCI_I2C1 Bus Clock */
    printf("USCI_I2C clock %d Hz\n", USCI_I2C_GetBusClockFreq(USCI1));

    /* Set USCI_I2C1 Slave Addresses */
    USCI_I2C_SetSlaveAddr(USCI1, 0, 0x15, USCI_I2C_GCMODE_DISABLE);   /* Slave Address : 0x15 */

    USCI_I2C_SetSlaveAddrMask(USCI1, 0, 0x01);

    USCI_I2C_ENABLE_PROT_INT(USCI1, (USCI_PROTIEN_ACKIEN_Msk | USCI_PROTIEN_NACKIEN_Msk | USCI_PROTIEN_STORIEN_Msk | USCI_PROTIEN_STARIEN_Msk));
    NVIC_EnableIRQ(USCI1_IRQn);
}

int main()
{
    uint32_t i;
    SYS_Init();

    /* Init USCI UART0 to 115200-8n1 for print message */
    USCI_UART_Open(USCI0, 115200);

    /*
        This sample code sets USCI_I2C bus clock to 100kHz. Then, Master accesses Slave with Byte Write
        and Byte Read operations, and check if the read data is equal to the programmed data.
    */

    printf("+-------------------------------------------------------+\n");
    printf("|          USCI_I2C Driver Sample Code(Slave)           |\n");
    printf("+-------------------------------------------------------+\n");

    /* Init USCI_I2C1 */
    USCI_I2C1_Init();

    /* USCI_I2C1 enter no address SLV mode */
    s_Event = SLAVE_ADDRESS_ACK;
    USCI_I2C_SET_CONTROL_REG(USCI1, USCI_I2C_CTL_AA);

    for (i = 0; i < 0x100; i++) {
        g_au8SlvData[i] = 0;
    }

    /* USCI_I2C1 function to Slave receive/transmit data */
    s_USCI_I2CSHandlerFn = USCI_I2C_SlaveTRx;

    printf("\n");
    printf("USCI_I2C1 Slave Mode is Running.\n");

    while(1);
}

/*** (C) COPYRIGHT 2016 Nuvoton Technology Corp. ***/
